from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class VitaminAUnits(Enum):
        """
            VitaminAUnits enumeration
        """
        
        InternationalUnit = 'InternationalUnit'
        """
            
        """
        

class VitaminADto:
    """
    A DTO representation of a VitaminA

    Attributes:
        value (float): The value of the VitaminA.
        unit (VitaminAUnits): The specific unit that the VitaminA value is representing.
    """

    def __init__(self, value: float, unit: VitaminAUnits):
        """
        Create a new DTO representation of a VitaminA

        Parameters:
            value (float): The value of the VitaminA.
            unit (VitaminAUnits): The specific unit that the VitaminA value is representing.
        """
        self.value: float = value
        """
        The value of the VitaminA
        """
        self.unit: VitaminAUnits = unit
        """
        The specific unit that the VitaminA value is representing
        """

    def to_json(self):
        """
        Get a VitaminA DTO JSON object representing the current unit.

        :return: JSON object represents VitaminA DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InternationalUnit"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of VitaminA DTO from a json representation.

        :param data: The VitaminA DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InternationalUnit"}
        :return: A new instance of VitaminADto.
        :rtype: VitaminADto
        """
        return VitaminADto(value=data["value"], unit=VitaminAUnits(data["unit"]))


class VitaminA(AbstractMeasure):
    """
    Vitamin A: 1 IU is the biological equivalent of 0.3 µg retinol, or of 0.6 µg beta-carotene.

    Args:
        value (float): The value.
        from_unit (VitaminAUnits): The VitaminA unit to create from, The default unit is InternationalUnit
    """
    def __init__(self, value: float, from_unit: VitaminAUnits = VitaminAUnits.InternationalUnit):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__international_units = None
        

    def convert(self, unit: VitaminAUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: VitaminAUnits = VitaminAUnits.InternationalUnit) -> VitaminADto:
        """
        Get a new instance of VitaminA DTO representing the current unit.

        :param hold_in_unit: The specific VitaminA unit to store the VitaminA value in the DTO representation.
        :type hold_in_unit: VitaminAUnits
        :return: A new instance of VitaminADto.
        :rtype: VitaminADto
        """
        return VitaminADto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: VitaminAUnits = VitaminAUnits.InternationalUnit):
        """
        Get a VitaminA DTO JSON object representing the current unit.

        :param hold_in_unit: The specific VitaminA unit to store the VitaminA value in the DTO representation.
        :type hold_in_unit: VitaminAUnits
        :return: JSON object represents VitaminA DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InternationalUnit"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(vitamin_a_dto: VitaminADto):
        """
        Obtain a new instance of VitaminA from a DTO unit object.

        :param vitamin_a_dto: The VitaminA DTO representation.
        :type vitamin_a_dto: VitaminADto
        :return: A new instance of VitaminA.
        :rtype: VitaminA
        """
        return VitaminA(vitamin_a_dto.value, vitamin_a_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of VitaminA from a DTO unit json representation.

        :param data: The VitaminA DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InternationalUnit"}
        :return: A new instance of VitaminA.
        :rtype: VitaminA
        """
        return VitaminA.from_dto(VitaminADto.from_json(data))

    def __convert_from_base(self, from_unit: VitaminAUnits) -> float:
        value = self._value
        
        if from_unit == VitaminAUnits.InternationalUnit:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VitaminAUnits) -> float:
        
        if to_unit == VitaminAUnits.InternationalUnit:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_international_units(international_units: float):
        """
        Create a new instance of VitaminA from a value in international_units.

        

        :param meters: The VitaminA value in international_units.
        :type international_units: float
        :return: A new instance of VitaminA.
        :rtype: VitaminA
        """
        return VitaminA(international_units, VitaminAUnits.InternationalUnit)

    
    @property
    def international_units(self) -> float:
        """
        
        """
        if self.__international_units != None:
            return self.__international_units
        self.__international_units = self.__convert_from_base(VitaminAUnits.InternationalUnit)
        return self.__international_units

    
    def to_string(self, unit: VitaminAUnits = VitaminAUnits.InternationalUnit, fractional_digits: int = None) -> str:
        """
        Format the VitaminA to a string.
        
        Note: the default format for VitaminA is InternationalUnit.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the VitaminA. Default is 'InternationalUnit'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == VitaminAUnits.InternationalUnit:
            return f"""{super()._truncate_fraction_digits(self.international_units, fractional_digits)} IU"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: VitaminAUnits = VitaminAUnits.InternationalUnit) -> str:
        """
        Get VitaminA unit abbreviation.
        Note! the default abbreviation for VitaminA is InternationalUnit.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VitaminAUnits.InternationalUnit:
            return """IU"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricCurrentDensityUnits(Enum):
        """
            ElectricCurrentDensityUnits enumeration
        """
        
        AmperePerSquareMeter = 'AmperePerSquareMeter'
        """
            
        """
        
        AmperePerSquareInch = 'AmperePerSquareInch'
        """
            
        """
        
        AmperePerSquareFoot = 'AmperePerSquareFoot'
        """
            
        """
        

class ElectricCurrentDensityDto:
    """
    A DTO representation of a ElectricCurrentDensity

    Attributes:
        value (float): The value of the ElectricCurrentDensity.
        unit (ElectricCurrentDensityUnits): The specific unit that the ElectricCurrentDensity value is representing.
    """

    def __init__(self, value: float, unit: ElectricCurrentDensityUnits):
        """
        Create a new DTO representation of a ElectricCurrentDensity

        Parameters:
            value (float): The value of the ElectricCurrentDensity.
            unit (ElectricCurrentDensityUnits): The specific unit that the ElectricCurrentDensity value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricCurrentDensity
        """
        self.unit: ElectricCurrentDensityUnits = unit
        """
        The specific unit that the ElectricCurrentDensity value is representing
        """

    def to_json(self):
        """
        Get a ElectricCurrentDensity DTO JSON object representing the current unit.

        :return: JSON object represents ElectricCurrentDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "AmperePerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricCurrentDensity DTO from a json representation.

        :param data: The ElectricCurrentDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "AmperePerSquareMeter"}
        :return: A new instance of ElectricCurrentDensityDto.
        :rtype: ElectricCurrentDensityDto
        """
        return ElectricCurrentDensityDto(value=data["value"], unit=ElectricCurrentDensityUnits(data["unit"]))


class ElectricCurrentDensity(AbstractMeasure):
    """
    In electromagnetism, current density is the electric current per unit area of cross section.

    Args:
        value (float): The value.
        from_unit (ElectricCurrentDensityUnits): The ElectricCurrentDensity unit to create from, The default unit is AmperePerSquareMeter
    """
    def __init__(self, value: float, from_unit: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__amperes_per_square_meter = None
        
        self.__amperes_per_square_inch = None
        
        self.__amperes_per_square_foot = None
        

    def convert(self, unit: ElectricCurrentDensityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter) -> ElectricCurrentDensityDto:
        """
        Get a new instance of ElectricCurrentDensity DTO representing the current unit.

        :param hold_in_unit: The specific ElectricCurrentDensity unit to store the ElectricCurrentDensity value in the DTO representation.
        :type hold_in_unit: ElectricCurrentDensityUnits
        :return: A new instance of ElectricCurrentDensityDto.
        :rtype: ElectricCurrentDensityDto
        """
        return ElectricCurrentDensityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter):
        """
        Get a ElectricCurrentDensity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricCurrentDensity unit to store the ElectricCurrentDensity value in the DTO representation.
        :type hold_in_unit: ElectricCurrentDensityUnits
        :return: JSON object represents ElectricCurrentDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "AmperePerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_current_density_dto: ElectricCurrentDensityDto):
        """
        Obtain a new instance of ElectricCurrentDensity from a DTO unit object.

        :param electric_current_density_dto: The ElectricCurrentDensity DTO representation.
        :type electric_current_density_dto: ElectricCurrentDensityDto
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(electric_current_density_dto.value, electric_current_density_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricCurrentDensity from a DTO unit json representation.

        :param data: The ElectricCurrentDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "AmperePerSquareMeter"}
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity.from_dto(ElectricCurrentDensityDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricCurrentDensityUnits) -> float:
        value = self._value
        
        if from_unit == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return (value)
        
        if from_unit == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return (value / 1.5500031000062000e3)
        
        if from_unit == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return (value / 1.0763910416709722e1)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricCurrentDensityUnits) -> float:
        
        if to_unit == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return (value)
        
        if to_unit == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return (value * 1.5500031000062000e3)
        
        if to_unit == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return (value * 1.0763910416709722e1)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_amperes_per_square_meter(amperes_per_square_meter: float):
        """
        Create a new instance of ElectricCurrentDensity from a value in amperes_per_square_meter.

        

        :param meters: The ElectricCurrentDensity value in amperes_per_square_meter.
        :type amperes_per_square_meter: float
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(amperes_per_square_meter, ElectricCurrentDensityUnits.AmperePerSquareMeter)

    
    @staticmethod
    def from_amperes_per_square_inch(amperes_per_square_inch: float):
        """
        Create a new instance of ElectricCurrentDensity from a value in amperes_per_square_inch.

        

        :param meters: The ElectricCurrentDensity value in amperes_per_square_inch.
        :type amperes_per_square_inch: float
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(amperes_per_square_inch, ElectricCurrentDensityUnits.AmperePerSquareInch)

    
    @staticmethod
    def from_amperes_per_square_foot(amperes_per_square_foot: float):
        """
        Create a new instance of ElectricCurrentDensity from a value in amperes_per_square_foot.

        

        :param meters: The ElectricCurrentDensity value in amperes_per_square_foot.
        :type amperes_per_square_foot: float
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(amperes_per_square_foot, ElectricCurrentDensityUnits.AmperePerSquareFoot)

    
    @property
    def amperes_per_square_meter(self) -> float:
        """
        
        """
        if self.__amperes_per_square_meter != None:
            return self.__amperes_per_square_meter
        self.__amperes_per_square_meter = self.__convert_from_base(ElectricCurrentDensityUnits.AmperePerSquareMeter)
        return self.__amperes_per_square_meter

    
    @property
    def amperes_per_square_inch(self) -> float:
        """
        
        """
        if self.__amperes_per_square_inch != None:
            return self.__amperes_per_square_inch
        self.__amperes_per_square_inch = self.__convert_from_base(ElectricCurrentDensityUnits.AmperePerSquareInch)
        return self.__amperes_per_square_inch

    
    @property
    def amperes_per_square_foot(self) -> float:
        """
        
        """
        if self.__amperes_per_square_foot != None:
            return self.__amperes_per_square_foot
        self.__amperes_per_square_foot = self.__convert_from_base(ElectricCurrentDensityUnits.AmperePerSquareFoot)
        return self.__amperes_per_square_foot

    
    def to_string(self, unit: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the ElectricCurrentDensity to a string.
        
        Note: the default format for ElectricCurrentDensity is AmperePerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricCurrentDensity. Default is 'AmperePerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_square_meter, fractional_digits)} A/m²"""
        
        if unit == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_square_inch, fractional_digits)} A/in²"""
        
        if unit == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_square_foot, fractional_digits)} A/ft²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter) -> str:
        """
        Get ElectricCurrentDensity unit abbreviation.
        Note! the default abbreviation for ElectricCurrentDensity is AmperePerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return """A/m²"""
        
        if unit_abbreviation == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return """A/in²"""
        
        if unit_abbreviation == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return """A/ft²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MassUnits(Enum):
        """
            MassUnits enumeration
        """
        
        Gram = 'Gram'
        """
            
        """
        
        Tonne = 'Tonne'
        """
            
        """
        
        ShortTon = 'ShortTon'
        """
            The short ton is a unit of mass equal to 2,000 pounds (907.18474 kg), that is most commonly used in the United States – known there simply as the ton.
        """
        
        LongTon = 'LongTon'
        """
            Long ton (weight ton or Imperial ton) is a unit of mass equal to 2,240 pounds (1,016 kg) and is the name for the unit called the "ton" in the avoirdupois or Imperial system of measurements that was used in the United Kingdom and several other Commonwealth countries before metrication.
        """
        
        Pound = 'Pound'
        """
            The pound or pound-mass (abbreviations: lb, lbm) is a unit of mass used in the imperial, United States customary and other systems of measurement. A number of different definitions have been used, the most common today being the international avoirdupois pound which is legally defined as exactly 0.45359237 kilograms, and which is divided into 16 avoirdupois ounces.
        """
        
        Ounce = 'Ounce'
        """
            The international avoirdupois ounce (abbreviated oz) is defined as exactly 28.349523125 g under the international yard and pound agreement of 1959, signed by the United States and countries of the Commonwealth of Nations. 16 oz make up an avoirdupois pound.
        """
        
        Slug = 'Slug'
        """
            The slug (abbreviation slug) is a unit of mass that is accelerated by 1 ft/s² when a force of one pound (lbf) is exerted on it.
        """
        
        Stone = 'Stone'
        """
            The stone (abbreviation st) is a unit of mass equal to 14 pounds avoirdupois (about 6.35 kilograms) used in Great Britain and Ireland for measuring human body weight.
        """
        
        ShortHundredweight = 'ShortHundredweight'
        """
            The short hundredweight (abbreviation cwt) is a unit of mass equal to 100 pounds in US and Canada. In British English, the short hundredweight is referred to as the "cental".
        """
        
        LongHundredweight = 'LongHundredweight'
        """
            The long or imperial hundredweight (abbreviation cwt) is a unit of mass equal to 112 pounds in US and Canada.
        """
        
        Grain = 'Grain'
        """
            A grain is a unit of measurement of mass, and in the troy weight, avoirdupois, and Apothecaries' system, equal to exactly 64.79891 milligrams.
        """
        
        SolarMass = 'SolarMass'
        """
            Solar mass is a ratio unit to the mass of the solar system star, the sun.
        """
        
        EarthMass = 'EarthMass'
        """
            Earth mass is a ratio unit to the mass of planet Earth.
        """
        
        Femtogram = 'Femtogram'
        """
            
        """
        
        Picogram = 'Picogram'
        """
            
        """
        
        Nanogram = 'Nanogram'
        """
            
        """
        
        Microgram = 'Microgram'
        """
            
        """
        
        Milligram = 'Milligram'
        """
            
        """
        
        Centigram = 'Centigram'
        """
            
        """
        
        Decigram = 'Decigram'
        """
            
        """
        
        Decagram = 'Decagram'
        """
            
        """
        
        Hectogram = 'Hectogram'
        """
            
        """
        
        Kilogram = 'Kilogram'
        """
            
        """
        
        Kilotonne = 'Kilotonne'
        """
            
        """
        
        Megatonne = 'Megatonne'
        """
            
        """
        
        Kilopound = 'Kilopound'
        """
            
        """
        
        Megapound = 'Megapound'
        """
            
        """
        

class MassDto:
    """
    A DTO representation of a Mass

    Attributes:
        value (float): The value of the Mass.
        unit (MassUnits): The specific unit that the Mass value is representing.
    """

    def __init__(self, value: float, unit: MassUnits):
        """
        Create a new DTO representation of a Mass

        Parameters:
            value (float): The value of the Mass.
            unit (MassUnits): The specific unit that the Mass value is representing.
        """
        self.value: float = value
        """
        The value of the Mass
        """
        self.unit: MassUnits = unit
        """
        The specific unit that the Mass value is representing
        """

    def to_json(self):
        """
        Get a Mass DTO JSON object representing the current unit.

        :return: JSON object represents Mass DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Kilogram"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Mass DTO from a json representation.

        :param data: The Mass DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Kilogram"}
        :return: A new instance of MassDto.
        :rtype: MassDto
        """
        return MassDto(value=data["value"], unit=MassUnits(data["unit"]))


class Mass(AbstractMeasure):
    """
    In physics, mass (from Greek μᾶζα "barley cake, lump [of dough]") is a property of a physical system or body, giving rise to the phenomena of the body's resistance to being accelerated by a force and the strength of its mutual gravitational attraction with other bodies. Instruments such as mass balances or scales use those phenomena to measure mass. The SI unit of mass is the kilogram (kg).

    Args:
        value (float): The value.
        from_unit (MassUnits): The Mass unit to create from, The default unit is Kilogram
    """
    def __init__(self, value: float, from_unit: MassUnits = MassUnits.Kilogram):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__grams = None
        
        self.__tonnes = None
        
        self.__short_tons = None
        
        self.__long_tons = None
        
        self.__pounds = None
        
        self.__ounces = None
        
        self.__slugs = None
        
        self.__stone = None
        
        self.__short_hundredweight = None
        
        self.__long_hundredweight = None
        
        self.__grains = None
        
        self.__solar_masses = None
        
        self.__earth_masses = None
        
        self.__femtograms = None
        
        self.__picograms = None
        
        self.__nanograms = None
        
        self.__micrograms = None
        
        self.__milligrams = None
        
        self.__centigrams = None
        
        self.__decigrams = None
        
        self.__decagrams = None
        
        self.__hectograms = None
        
        self.__kilograms = None
        
        self.__kilotonnes = None
        
        self.__megatonnes = None
        
        self.__kilopounds = None
        
        self.__megapounds = None
        

    def convert(self, unit: MassUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MassUnits = MassUnits.Kilogram) -> MassDto:
        """
        Get a new instance of Mass DTO representing the current unit.

        :param hold_in_unit: The specific Mass unit to store the Mass value in the DTO representation.
        :type hold_in_unit: MassUnits
        :return: A new instance of MassDto.
        :rtype: MassDto
        """
        return MassDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MassUnits = MassUnits.Kilogram):
        """
        Get a Mass DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Mass unit to store the Mass value in the DTO representation.
        :type hold_in_unit: MassUnits
        :return: JSON object represents Mass DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Kilogram"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(mass_dto: MassDto):
        """
        Obtain a new instance of Mass from a DTO unit object.

        :param mass_dto: The Mass DTO representation.
        :type mass_dto: MassDto
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(mass_dto.value, mass_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Mass from a DTO unit json representation.

        :param data: The Mass DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Kilogram"}
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass.from_dto(MassDto.from_json(data))

    def __convert_from_base(self, from_unit: MassUnits) -> float:
        value = self._value
        
        if from_unit == MassUnits.Gram:
            return (value * 1e3)
        
        if from_unit == MassUnits.Tonne:
            return (value / 1e3)
        
        if from_unit == MassUnits.ShortTon:
            return (value / 9.0718474e2)
        
        if from_unit == MassUnits.LongTon:
            return (value / 1.0160469088e3)
        
        if from_unit == MassUnits.Pound:
            return (value / 0.45359237)
        
        if from_unit == MassUnits.Ounce:
            return (value / 0.028349523125)
        
        if from_unit == MassUnits.Slug:
            return (value * 6.852176556196105e-2)
        
        if from_unit == MassUnits.Stone:
            return (value * 0.1574731728702698)
        
        if from_unit == MassUnits.ShortHundredweight:
            return (value * 0.022046226218487758)
        
        if from_unit == MassUnits.LongHundredweight:
            return (value * 0.01968413055222121)
        
        if from_unit == MassUnits.Grain:
            return (value * 15432.358352941431)
        
        if from_unit == MassUnits.SolarMass:
            return (value / 1.98947e30)
        
        if from_unit == MassUnits.EarthMass:
            return (value / 5.9722e+24)
        
        if from_unit == MassUnits.Femtogram:
            return ((value * 1e3) / 1e-15)
        
        if from_unit == MassUnits.Picogram:
            return ((value * 1e3) / 1e-12)
        
        if from_unit == MassUnits.Nanogram:
            return ((value * 1e3) / 1e-09)
        
        if from_unit == MassUnits.Microgram:
            return ((value * 1e3) / 1e-06)
        
        if from_unit == MassUnits.Milligram:
            return ((value * 1e3) / 0.001)
        
        if from_unit == MassUnits.Centigram:
            return ((value * 1e3) / 0.01)
        
        if from_unit == MassUnits.Decigram:
            return ((value * 1e3) / 0.1)
        
        if from_unit == MassUnits.Decagram:
            return ((value * 1e3) / 10.0)
        
        if from_unit == MassUnits.Hectogram:
            return ((value * 1e3) / 100.0)
        
        if from_unit == MassUnits.Kilogram:
            return ((value * 1e3) / 1000.0)
        
        if from_unit == MassUnits.Kilotonne:
            return ((value / 1e3) / 1000.0)
        
        if from_unit == MassUnits.Megatonne:
            return ((value / 1e3) / 1000000.0)
        
        if from_unit == MassUnits.Kilopound:
            return ((value / 0.45359237) / 1000.0)
        
        if from_unit == MassUnits.Megapound:
            return ((value / 0.45359237) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MassUnits) -> float:
        
        if to_unit == MassUnits.Gram:
            return (value / 1e3)
        
        if to_unit == MassUnits.Tonne:
            return (value * 1e3)
        
        if to_unit == MassUnits.ShortTon:
            return (value * 9.0718474e2)
        
        if to_unit == MassUnits.LongTon:
            return (value * 1.0160469088e3)
        
        if to_unit == MassUnits.Pound:
            return (value * 0.45359237)
        
        if to_unit == MassUnits.Ounce:
            return (value * 0.028349523125)
        
        if to_unit == MassUnits.Slug:
            return (value / 6.852176556196105e-2)
        
        if to_unit == MassUnits.Stone:
            return (value / 0.1574731728702698)
        
        if to_unit == MassUnits.ShortHundredweight:
            return (value / 0.022046226218487758)
        
        if to_unit == MassUnits.LongHundredweight:
            return (value / 0.01968413055222121)
        
        if to_unit == MassUnits.Grain:
            return (value / 15432.358352941431)
        
        if to_unit == MassUnits.SolarMass:
            return (value * 1.98947e30)
        
        if to_unit == MassUnits.EarthMass:
            return (value * 5.9722e+24)
        
        if to_unit == MassUnits.Femtogram:
            return ((value / 1e3) * 1e-15)
        
        if to_unit == MassUnits.Picogram:
            return ((value / 1e3) * 1e-12)
        
        if to_unit == MassUnits.Nanogram:
            return ((value / 1e3) * 1e-09)
        
        if to_unit == MassUnits.Microgram:
            return ((value / 1e3) * 1e-06)
        
        if to_unit == MassUnits.Milligram:
            return ((value / 1e3) * 0.001)
        
        if to_unit == MassUnits.Centigram:
            return ((value / 1e3) * 0.01)
        
        if to_unit == MassUnits.Decigram:
            return ((value / 1e3) * 0.1)
        
        if to_unit == MassUnits.Decagram:
            return ((value / 1e3) * 10.0)
        
        if to_unit == MassUnits.Hectogram:
            return ((value / 1e3) * 100.0)
        
        if to_unit == MassUnits.Kilogram:
            return ((value / 1e3) * 1000.0)
        
        if to_unit == MassUnits.Kilotonne:
            return ((value * 1e3) * 1000.0)
        
        if to_unit == MassUnits.Megatonne:
            return ((value * 1e3) * 1000000.0)
        
        if to_unit == MassUnits.Kilopound:
            return ((value * 0.45359237) * 1000.0)
        
        if to_unit == MassUnits.Megapound:
            return ((value * 0.45359237) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_grams(grams: float):
        """
        Create a new instance of Mass from a value in grams.

        

        :param meters: The Mass value in grams.
        :type grams: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(grams, MassUnits.Gram)

    
    @staticmethod
    def from_tonnes(tonnes: float):
        """
        Create a new instance of Mass from a value in tonnes.

        

        :param meters: The Mass value in tonnes.
        :type tonnes: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(tonnes, MassUnits.Tonne)

    
    @staticmethod
    def from_short_tons(short_tons: float):
        """
        Create a new instance of Mass from a value in short_tons.

        The short ton is a unit of mass equal to 2,000 pounds (907.18474 kg), that is most commonly used in the United States – known there simply as the ton.

        :param meters: The Mass value in short_tons.
        :type short_tons: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(short_tons, MassUnits.ShortTon)

    
    @staticmethod
    def from_long_tons(long_tons: float):
        """
        Create a new instance of Mass from a value in long_tons.

        Long ton (weight ton or Imperial ton) is a unit of mass equal to 2,240 pounds (1,016 kg) and is the name for the unit called the "ton" in the avoirdupois or Imperial system of measurements that was used in the United Kingdom and several other Commonwealth countries before metrication.

        :param meters: The Mass value in long_tons.
        :type long_tons: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(long_tons, MassUnits.LongTon)

    
    @staticmethod
    def from_pounds(pounds: float):
        """
        Create a new instance of Mass from a value in pounds.

        The pound or pound-mass (abbreviations: lb, lbm) is a unit of mass used in the imperial, United States customary and other systems of measurement. A number of different definitions have been used, the most common today being the international avoirdupois pound which is legally defined as exactly 0.45359237 kilograms, and which is divided into 16 avoirdupois ounces.

        :param meters: The Mass value in pounds.
        :type pounds: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(pounds, MassUnits.Pound)

    
    @staticmethod
    def from_ounces(ounces: float):
        """
        Create a new instance of Mass from a value in ounces.

        The international avoirdupois ounce (abbreviated oz) is defined as exactly 28.349523125 g under the international yard and pound agreement of 1959, signed by the United States and countries of the Commonwealth of Nations. 16 oz make up an avoirdupois pound.

        :param meters: The Mass value in ounces.
        :type ounces: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(ounces, MassUnits.Ounce)

    
    @staticmethod
    def from_slugs(slugs: float):
        """
        Create a new instance of Mass from a value in slugs.

        The slug (abbreviation slug) is a unit of mass that is accelerated by 1 ft/s² when a force of one pound (lbf) is exerted on it.

        :param meters: The Mass value in slugs.
        :type slugs: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(slugs, MassUnits.Slug)

    
    @staticmethod
    def from_stone(stone: float):
        """
        Create a new instance of Mass from a value in stone.

        The stone (abbreviation st) is a unit of mass equal to 14 pounds avoirdupois (about 6.35 kilograms) used in Great Britain and Ireland for measuring human body weight.

        :param meters: The Mass value in stone.
        :type stone: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(stone, MassUnits.Stone)

    
    @staticmethod
    def from_short_hundredweight(short_hundredweight: float):
        """
        Create a new instance of Mass from a value in short_hundredweight.

        The short hundredweight (abbreviation cwt) is a unit of mass equal to 100 pounds in US and Canada. In British English, the short hundredweight is referred to as the "cental".

        :param meters: The Mass value in short_hundredweight.
        :type short_hundredweight: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(short_hundredweight, MassUnits.ShortHundredweight)

    
    @staticmethod
    def from_long_hundredweight(long_hundredweight: float):
        """
        Create a new instance of Mass from a value in long_hundredweight.

        The long or imperial hundredweight (abbreviation cwt) is a unit of mass equal to 112 pounds in US and Canada.

        :param meters: The Mass value in long_hundredweight.
        :type long_hundredweight: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(long_hundredweight, MassUnits.LongHundredweight)

    
    @staticmethod
    def from_grains(grains: float):
        """
        Create a new instance of Mass from a value in grains.

        A grain is a unit of measurement of mass, and in the troy weight, avoirdupois, and Apothecaries' system, equal to exactly 64.79891 milligrams.

        :param meters: The Mass value in grains.
        :type grains: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(grains, MassUnits.Grain)

    
    @staticmethod
    def from_solar_masses(solar_masses: float):
        """
        Create a new instance of Mass from a value in solar_masses.

        Solar mass is a ratio unit to the mass of the solar system star, the sun.

        :param meters: The Mass value in solar_masses.
        :type solar_masses: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(solar_masses, MassUnits.SolarMass)

    
    @staticmethod
    def from_earth_masses(earth_masses: float):
        """
        Create a new instance of Mass from a value in earth_masses.

        Earth mass is a ratio unit to the mass of planet Earth.

        :param meters: The Mass value in earth_masses.
        :type earth_masses: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(earth_masses, MassUnits.EarthMass)

    
    @staticmethod
    def from_femtograms(femtograms: float):
        """
        Create a new instance of Mass from a value in femtograms.

        

        :param meters: The Mass value in femtograms.
        :type femtograms: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(femtograms, MassUnits.Femtogram)

    
    @staticmethod
    def from_picograms(picograms: float):
        """
        Create a new instance of Mass from a value in picograms.

        

        :param meters: The Mass value in picograms.
        :type picograms: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(picograms, MassUnits.Picogram)

    
    @staticmethod
    def from_nanograms(nanograms: float):
        """
        Create a new instance of Mass from a value in nanograms.

        

        :param meters: The Mass value in nanograms.
        :type nanograms: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(nanograms, MassUnits.Nanogram)

    
    @staticmethod
    def from_micrograms(micrograms: float):
        """
        Create a new instance of Mass from a value in micrograms.

        

        :param meters: The Mass value in micrograms.
        :type micrograms: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(micrograms, MassUnits.Microgram)

    
    @staticmethod
    def from_milligrams(milligrams: float):
        """
        Create a new instance of Mass from a value in milligrams.

        

        :param meters: The Mass value in milligrams.
        :type milligrams: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(milligrams, MassUnits.Milligram)

    
    @staticmethod
    def from_centigrams(centigrams: float):
        """
        Create a new instance of Mass from a value in centigrams.

        

        :param meters: The Mass value in centigrams.
        :type centigrams: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(centigrams, MassUnits.Centigram)

    
    @staticmethod
    def from_decigrams(decigrams: float):
        """
        Create a new instance of Mass from a value in decigrams.

        

        :param meters: The Mass value in decigrams.
        :type decigrams: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(decigrams, MassUnits.Decigram)

    
    @staticmethod
    def from_decagrams(decagrams: float):
        """
        Create a new instance of Mass from a value in decagrams.

        

        :param meters: The Mass value in decagrams.
        :type decagrams: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(decagrams, MassUnits.Decagram)

    
    @staticmethod
    def from_hectograms(hectograms: float):
        """
        Create a new instance of Mass from a value in hectograms.

        

        :param meters: The Mass value in hectograms.
        :type hectograms: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(hectograms, MassUnits.Hectogram)

    
    @staticmethod
    def from_kilograms(kilograms: float):
        """
        Create a new instance of Mass from a value in kilograms.

        

        :param meters: The Mass value in kilograms.
        :type kilograms: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(kilograms, MassUnits.Kilogram)

    
    @staticmethod
    def from_kilotonnes(kilotonnes: float):
        """
        Create a new instance of Mass from a value in kilotonnes.

        

        :param meters: The Mass value in kilotonnes.
        :type kilotonnes: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(kilotonnes, MassUnits.Kilotonne)

    
    @staticmethod
    def from_megatonnes(megatonnes: float):
        """
        Create a new instance of Mass from a value in megatonnes.

        

        :param meters: The Mass value in megatonnes.
        :type megatonnes: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(megatonnes, MassUnits.Megatonne)

    
    @staticmethod
    def from_kilopounds(kilopounds: float):
        """
        Create a new instance of Mass from a value in kilopounds.

        

        :param meters: The Mass value in kilopounds.
        :type kilopounds: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(kilopounds, MassUnits.Kilopound)

    
    @staticmethod
    def from_megapounds(megapounds: float):
        """
        Create a new instance of Mass from a value in megapounds.

        

        :param meters: The Mass value in megapounds.
        :type megapounds: float
        :return: A new instance of Mass.
        :rtype: Mass
        """
        return Mass(megapounds, MassUnits.Megapound)

    
    @property
    def grams(self) -> float:
        """
        
        """
        if self.__grams != None:
            return self.__grams
        self.__grams = self.__convert_from_base(MassUnits.Gram)
        return self.__grams

    
    @property
    def tonnes(self) -> float:
        """
        
        """
        if self.__tonnes != None:
            return self.__tonnes
        self.__tonnes = self.__convert_from_base(MassUnits.Tonne)
        return self.__tonnes

    
    @property
    def short_tons(self) -> float:
        """
        The short ton is a unit of mass equal to 2,000 pounds (907.18474 kg), that is most commonly used in the United States – known there simply as the ton.
        """
        if self.__short_tons != None:
            return self.__short_tons
        self.__short_tons = self.__convert_from_base(MassUnits.ShortTon)
        return self.__short_tons

    
    @property
    def long_tons(self) -> float:
        """
        Long ton (weight ton or Imperial ton) is a unit of mass equal to 2,240 pounds (1,016 kg) and is the name for the unit called the "ton" in the avoirdupois or Imperial system of measurements that was used in the United Kingdom and several other Commonwealth countries before metrication.
        """
        if self.__long_tons != None:
            return self.__long_tons
        self.__long_tons = self.__convert_from_base(MassUnits.LongTon)
        return self.__long_tons

    
    @property
    def pounds(self) -> float:
        """
        The pound or pound-mass (abbreviations: lb, lbm) is a unit of mass used in the imperial, United States customary and other systems of measurement. A number of different definitions have been used, the most common today being the international avoirdupois pound which is legally defined as exactly 0.45359237 kilograms, and which is divided into 16 avoirdupois ounces.
        """
        if self.__pounds != None:
            return self.__pounds
        self.__pounds = self.__convert_from_base(MassUnits.Pound)
        return self.__pounds

    
    @property
    def ounces(self) -> float:
        """
        The international avoirdupois ounce (abbreviated oz) is defined as exactly 28.349523125 g under the international yard and pound agreement of 1959, signed by the United States and countries of the Commonwealth of Nations. 16 oz make up an avoirdupois pound.
        """
        if self.__ounces != None:
            return self.__ounces
        self.__ounces = self.__convert_from_base(MassUnits.Ounce)
        return self.__ounces

    
    @property
    def slugs(self) -> float:
        """
        The slug (abbreviation slug) is a unit of mass that is accelerated by 1 ft/s² when a force of one pound (lbf) is exerted on it.
        """
        if self.__slugs != None:
            return self.__slugs
        self.__slugs = self.__convert_from_base(MassUnits.Slug)
        return self.__slugs

    
    @property
    def stone(self) -> float:
        """
        The stone (abbreviation st) is a unit of mass equal to 14 pounds avoirdupois (about 6.35 kilograms) used in Great Britain and Ireland for measuring human body weight.
        """
        if self.__stone != None:
            return self.__stone
        self.__stone = self.__convert_from_base(MassUnits.Stone)
        return self.__stone

    
    @property
    def short_hundredweight(self) -> float:
        """
        The short hundredweight (abbreviation cwt) is a unit of mass equal to 100 pounds in US and Canada. In British English, the short hundredweight is referred to as the "cental".
        """
        if self.__short_hundredweight != None:
            return self.__short_hundredweight
        self.__short_hundredweight = self.__convert_from_base(MassUnits.ShortHundredweight)
        return self.__short_hundredweight

    
    @property
    def long_hundredweight(self) -> float:
        """
        The long or imperial hundredweight (abbreviation cwt) is a unit of mass equal to 112 pounds in US and Canada.
        """
        if self.__long_hundredweight != None:
            return self.__long_hundredweight
        self.__long_hundredweight = self.__convert_from_base(MassUnits.LongHundredweight)
        return self.__long_hundredweight

    
    @property
    def grains(self) -> float:
        """
        A grain is a unit of measurement of mass, and in the troy weight, avoirdupois, and Apothecaries' system, equal to exactly 64.79891 milligrams.
        """
        if self.__grains != None:
            return self.__grains
        self.__grains = self.__convert_from_base(MassUnits.Grain)
        return self.__grains

    
    @property
    def solar_masses(self) -> float:
        """
        Solar mass is a ratio unit to the mass of the solar system star, the sun.
        """
        if self.__solar_masses != None:
            return self.__solar_masses
        self.__solar_masses = self.__convert_from_base(MassUnits.SolarMass)
        return self.__solar_masses

    
    @property
    def earth_masses(self) -> float:
        """
        Earth mass is a ratio unit to the mass of planet Earth.
        """
        if self.__earth_masses != None:
            return self.__earth_masses
        self.__earth_masses = self.__convert_from_base(MassUnits.EarthMass)
        return self.__earth_masses

    
    @property
    def femtograms(self) -> float:
        """
        
        """
        if self.__femtograms != None:
            return self.__femtograms
        self.__femtograms = self.__convert_from_base(MassUnits.Femtogram)
        return self.__femtograms

    
    @property
    def picograms(self) -> float:
        """
        
        """
        if self.__picograms != None:
            return self.__picograms
        self.__picograms = self.__convert_from_base(MassUnits.Picogram)
        return self.__picograms

    
    @property
    def nanograms(self) -> float:
        """
        
        """
        if self.__nanograms != None:
            return self.__nanograms
        self.__nanograms = self.__convert_from_base(MassUnits.Nanogram)
        return self.__nanograms

    
    @property
    def micrograms(self) -> float:
        """
        
        """
        if self.__micrograms != None:
            return self.__micrograms
        self.__micrograms = self.__convert_from_base(MassUnits.Microgram)
        return self.__micrograms

    
    @property
    def milligrams(self) -> float:
        """
        
        """
        if self.__milligrams != None:
            return self.__milligrams
        self.__milligrams = self.__convert_from_base(MassUnits.Milligram)
        return self.__milligrams

    
    @property
    def centigrams(self) -> float:
        """
        
        """
        if self.__centigrams != None:
            return self.__centigrams
        self.__centigrams = self.__convert_from_base(MassUnits.Centigram)
        return self.__centigrams

    
    @property
    def decigrams(self) -> float:
        """
        
        """
        if self.__decigrams != None:
            return self.__decigrams
        self.__decigrams = self.__convert_from_base(MassUnits.Decigram)
        return self.__decigrams

    
    @property
    def decagrams(self) -> float:
        """
        
        """
        if self.__decagrams != None:
            return self.__decagrams
        self.__decagrams = self.__convert_from_base(MassUnits.Decagram)
        return self.__decagrams

    
    @property
    def hectograms(self) -> float:
        """
        
        """
        if self.__hectograms != None:
            return self.__hectograms
        self.__hectograms = self.__convert_from_base(MassUnits.Hectogram)
        return self.__hectograms

    
    @property
    def kilograms(self) -> float:
        """
        
        """
        if self.__kilograms != None:
            return self.__kilograms
        self.__kilograms = self.__convert_from_base(MassUnits.Kilogram)
        return self.__kilograms

    
    @property
    def kilotonnes(self) -> float:
        """
        
        """
        if self.__kilotonnes != None:
            return self.__kilotonnes
        self.__kilotonnes = self.__convert_from_base(MassUnits.Kilotonne)
        return self.__kilotonnes

    
    @property
    def megatonnes(self) -> float:
        """
        
        """
        if self.__megatonnes != None:
            return self.__megatonnes
        self.__megatonnes = self.__convert_from_base(MassUnits.Megatonne)
        return self.__megatonnes

    
    @property
    def kilopounds(self) -> float:
        """
        
        """
        if self.__kilopounds != None:
            return self.__kilopounds
        self.__kilopounds = self.__convert_from_base(MassUnits.Kilopound)
        return self.__kilopounds

    
    @property
    def megapounds(self) -> float:
        """
        
        """
        if self.__megapounds != None:
            return self.__megapounds
        self.__megapounds = self.__convert_from_base(MassUnits.Megapound)
        return self.__megapounds

    
    def to_string(self, unit: MassUnits = MassUnits.Kilogram, fractional_digits: int = None) -> str:
        """
        Format the Mass to a string.
        
        Note: the default format for Mass is Kilogram.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Mass. Default is 'Kilogram'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MassUnits.Gram:
            return f"""{super()._truncate_fraction_digits(self.grams, fractional_digits)} g"""
        
        if unit == MassUnits.Tonne:
            return f"""{super()._truncate_fraction_digits(self.tonnes, fractional_digits)} t"""
        
        if unit == MassUnits.ShortTon:
            return f"""{super()._truncate_fraction_digits(self.short_tons, fractional_digits)} t (short)"""
        
        if unit == MassUnits.LongTon:
            return f"""{super()._truncate_fraction_digits(self.long_tons, fractional_digits)} long tn"""
        
        if unit == MassUnits.Pound:
            return f"""{super()._truncate_fraction_digits(self.pounds, fractional_digits)} lb"""
        
        if unit == MassUnits.Ounce:
            return f"""{super()._truncate_fraction_digits(self.ounces, fractional_digits)} oz"""
        
        if unit == MassUnits.Slug:
            return f"""{super()._truncate_fraction_digits(self.slugs, fractional_digits)} slug"""
        
        if unit == MassUnits.Stone:
            return f"""{super()._truncate_fraction_digits(self.stone, fractional_digits)} st"""
        
        if unit == MassUnits.ShortHundredweight:
            return f"""{super()._truncate_fraction_digits(self.short_hundredweight, fractional_digits)} cwt"""
        
        if unit == MassUnits.LongHundredweight:
            return f"""{super()._truncate_fraction_digits(self.long_hundredweight, fractional_digits)} cwt"""
        
        if unit == MassUnits.Grain:
            return f"""{super()._truncate_fraction_digits(self.grains, fractional_digits)} gr"""
        
        if unit == MassUnits.SolarMass:
            return f"""{super()._truncate_fraction_digits(self.solar_masses, fractional_digits)} M☉"""
        
        if unit == MassUnits.EarthMass:
            return f"""{super()._truncate_fraction_digits(self.earth_masses, fractional_digits)} em"""
        
        if unit == MassUnits.Femtogram:
            return f"""{super()._truncate_fraction_digits(self.femtograms, fractional_digits)} fg"""
        
        if unit == MassUnits.Picogram:
            return f"""{super()._truncate_fraction_digits(self.picograms, fractional_digits)} pg"""
        
        if unit == MassUnits.Nanogram:
            return f"""{super()._truncate_fraction_digits(self.nanograms, fractional_digits)} ng"""
        
        if unit == MassUnits.Microgram:
            return f"""{super()._truncate_fraction_digits(self.micrograms, fractional_digits)} μg"""
        
        if unit == MassUnits.Milligram:
            return f"""{super()._truncate_fraction_digits(self.milligrams, fractional_digits)} mg"""
        
        if unit == MassUnits.Centigram:
            return f"""{super()._truncate_fraction_digits(self.centigrams, fractional_digits)} cg"""
        
        if unit == MassUnits.Decigram:
            return f"""{super()._truncate_fraction_digits(self.decigrams, fractional_digits)} dg"""
        
        if unit == MassUnits.Decagram:
            return f"""{super()._truncate_fraction_digits(self.decagrams, fractional_digits)} dag"""
        
        if unit == MassUnits.Hectogram:
            return f"""{super()._truncate_fraction_digits(self.hectograms, fractional_digits)} hg"""
        
        if unit == MassUnits.Kilogram:
            return f"""{super()._truncate_fraction_digits(self.kilograms, fractional_digits)} kg"""
        
        if unit == MassUnits.Kilotonne:
            return f"""{super()._truncate_fraction_digits(self.kilotonnes, fractional_digits)} kt"""
        
        if unit == MassUnits.Megatonne:
            return f"""{super()._truncate_fraction_digits(self.megatonnes, fractional_digits)} Mt"""
        
        if unit == MassUnits.Kilopound:
            return f"""{super()._truncate_fraction_digits(self.kilopounds, fractional_digits)} klb"""
        
        if unit == MassUnits.Megapound:
            return f"""{super()._truncate_fraction_digits(self.megapounds, fractional_digits)} Mlb"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MassUnits = MassUnits.Kilogram) -> str:
        """
        Get Mass unit abbreviation.
        Note! the default abbreviation for Mass is Kilogram.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MassUnits.Gram:
            return """g"""
        
        if unit_abbreviation == MassUnits.Tonne:
            return """t"""
        
        if unit_abbreviation == MassUnits.ShortTon:
            return """t (short)"""
        
        if unit_abbreviation == MassUnits.LongTon:
            return """long tn"""
        
        if unit_abbreviation == MassUnits.Pound:
            return """lb"""
        
        if unit_abbreviation == MassUnits.Ounce:
            return """oz"""
        
        if unit_abbreviation == MassUnits.Slug:
            return """slug"""
        
        if unit_abbreviation == MassUnits.Stone:
            return """st"""
        
        if unit_abbreviation == MassUnits.ShortHundredweight:
            return """cwt"""
        
        if unit_abbreviation == MassUnits.LongHundredweight:
            return """cwt"""
        
        if unit_abbreviation == MassUnits.Grain:
            return """gr"""
        
        if unit_abbreviation == MassUnits.SolarMass:
            return """M☉"""
        
        if unit_abbreviation == MassUnits.EarthMass:
            return """em"""
        
        if unit_abbreviation == MassUnits.Femtogram:
            return """fg"""
        
        if unit_abbreviation == MassUnits.Picogram:
            return """pg"""
        
        if unit_abbreviation == MassUnits.Nanogram:
            return """ng"""
        
        if unit_abbreviation == MassUnits.Microgram:
            return """μg"""
        
        if unit_abbreviation == MassUnits.Milligram:
            return """mg"""
        
        if unit_abbreviation == MassUnits.Centigram:
            return """cg"""
        
        if unit_abbreviation == MassUnits.Decigram:
            return """dg"""
        
        if unit_abbreviation == MassUnits.Decagram:
            return """dag"""
        
        if unit_abbreviation == MassUnits.Hectogram:
            return """hg"""
        
        if unit_abbreviation == MassUnits.Kilogram:
            return """kg"""
        
        if unit_abbreviation == MassUnits.Kilotonne:
            return """kt"""
        
        if unit_abbreviation == MassUnits.Megatonne:
            return """Mt"""
        
        if unit_abbreviation == MassUnits.Kilopound:
            return """klb"""
        
        if unit_abbreviation == MassUnits.Megapound:
            return """Mlb"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricChargeUnits(Enum):
        """
            ElectricChargeUnits enumeration
        """
        
        Coulomb = 'Coulomb'
        """
            
        """
        
        AmpereHour = 'AmpereHour'
        """
            
        """
        
        Picocoulomb = 'Picocoulomb'
        """
            
        """
        
        Nanocoulomb = 'Nanocoulomb'
        """
            
        """
        
        Microcoulomb = 'Microcoulomb'
        """
            
        """
        
        Millicoulomb = 'Millicoulomb'
        """
            
        """
        
        Kilocoulomb = 'Kilocoulomb'
        """
            
        """
        
        Megacoulomb = 'Megacoulomb'
        """
            
        """
        
        MilliampereHour = 'MilliampereHour'
        """
            
        """
        
        KiloampereHour = 'KiloampereHour'
        """
            
        """
        
        MegaampereHour = 'MegaampereHour'
        """
            
        """
        

class ElectricChargeDto:
    """
    A DTO representation of a ElectricCharge

    Attributes:
        value (float): The value of the ElectricCharge.
        unit (ElectricChargeUnits): The specific unit that the ElectricCharge value is representing.
    """

    def __init__(self, value: float, unit: ElectricChargeUnits):
        """
        Create a new DTO representation of a ElectricCharge

        Parameters:
            value (float): The value of the ElectricCharge.
            unit (ElectricChargeUnits): The specific unit that the ElectricCharge value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricCharge
        """
        self.unit: ElectricChargeUnits = unit
        """
        The specific unit that the ElectricCharge value is representing
        """

    def to_json(self):
        """
        Get a ElectricCharge DTO JSON object representing the current unit.

        :return: JSON object represents ElectricCharge DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Coulomb"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricCharge DTO from a json representation.

        :param data: The ElectricCharge DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Coulomb"}
        :return: A new instance of ElectricChargeDto.
        :rtype: ElectricChargeDto
        """
        return ElectricChargeDto(value=data["value"], unit=ElectricChargeUnits(data["unit"]))


class ElectricCharge(AbstractMeasure):
    """
    Electric charge is the physical property of matter that causes it to experience a force when placed in an electromagnetic field.

    Args:
        value (float): The value.
        from_unit (ElectricChargeUnits): The ElectricCharge unit to create from, The default unit is Coulomb
    """
    def __init__(self, value: float, from_unit: ElectricChargeUnits = ElectricChargeUnits.Coulomb):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__coulombs = None
        
        self.__ampere_hours = None
        
        self.__picocoulombs = None
        
        self.__nanocoulombs = None
        
        self.__microcoulombs = None
        
        self.__millicoulombs = None
        
        self.__kilocoulombs = None
        
        self.__megacoulombs = None
        
        self.__milliampere_hours = None
        
        self.__kiloampere_hours = None
        
        self.__megaampere_hours = None
        

    def convert(self, unit: ElectricChargeUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricChargeUnits = ElectricChargeUnits.Coulomb) -> ElectricChargeDto:
        """
        Get a new instance of ElectricCharge DTO representing the current unit.

        :param hold_in_unit: The specific ElectricCharge unit to store the ElectricCharge value in the DTO representation.
        :type hold_in_unit: ElectricChargeUnits
        :return: A new instance of ElectricChargeDto.
        :rtype: ElectricChargeDto
        """
        return ElectricChargeDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricChargeUnits = ElectricChargeUnits.Coulomb):
        """
        Get a ElectricCharge DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricCharge unit to store the ElectricCharge value in the DTO representation.
        :type hold_in_unit: ElectricChargeUnits
        :return: JSON object represents ElectricCharge DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Coulomb"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_charge_dto: ElectricChargeDto):
        """
        Obtain a new instance of ElectricCharge from a DTO unit object.

        :param electric_charge_dto: The ElectricCharge DTO representation.
        :type electric_charge_dto: ElectricChargeDto
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(electric_charge_dto.value, electric_charge_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricCharge from a DTO unit json representation.

        :param data: The ElectricCharge DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Coulomb"}
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge.from_dto(ElectricChargeDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricChargeUnits) -> float:
        value = self._value
        
        if from_unit == ElectricChargeUnits.Coulomb:
            return (value)
        
        if from_unit == ElectricChargeUnits.AmpereHour:
            return (value * 2.77777777777e-4)
        
        if from_unit == ElectricChargeUnits.Picocoulomb:
            return ((value) / 1e-12)
        
        if from_unit == ElectricChargeUnits.Nanocoulomb:
            return ((value) / 1e-09)
        
        if from_unit == ElectricChargeUnits.Microcoulomb:
            return ((value) / 1e-06)
        
        if from_unit == ElectricChargeUnits.Millicoulomb:
            return ((value) / 0.001)
        
        if from_unit == ElectricChargeUnits.Kilocoulomb:
            return ((value) / 1000.0)
        
        if from_unit == ElectricChargeUnits.Megacoulomb:
            return ((value) / 1000000.0)
        
        if from_unit == ElectricChargeUnits.MilliampereHour:
            return ((value * 2.77777777777e-4) / 0.001)
        
        if from_unit == ElectricChargeUnits.KiloampereHour:
            return ((value * 2.77777777777e-4) / 1000.0)
        
        if from_unit == ElectricChargeUnits.MegaampereHour:
            return ((value * 2.77777777777e-4) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricChargeUnits) -> float:
        
        if to_unit == ElectricChargeUnits.Coulomb:
            return (value)
        
        if to_unit == ElectricChargeUnits.AmpereHour:
            return (value / 2.77777777777e-4)
        
        if to_unit == ElectricChargeUnits.Picocoulomb:
            return ((value) * 1e-12)
        
        if to_unit == ElectricChargeUnits.Nanocoulomb:
            return ((value) * 1e-09)
        
        if to_unit == ElectricChargeUnits.Microcoulomb:
            return ((value) * 1e-06)
        
        if to_unit == ElectricChargeUnits.Millicoulomb:
            return ((value) * 0.001)
        
        if to_unit == ElectricChargeUnits.Kilocoulomb:
            return ((value) * 1000.0)
        
        if to_unit == ElectricChargeUnits.Megacoulomb:
            return ((value) * 1000000.0)
        
        if to_unit == ElectricChargeUnits.MilliampereHour:
            return ((value / 2.77777777777e-4) * 0.001)
        
        if to_unit == ElectricChargeUnits.KiloampereHour:
            return ((value / 2.77777777777e-4) * 1000.0)
        
        if to_unit == ElectricChargeUnits.MegaampereHour:
            return ((value / 2.77777777777e-4) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_coulombs(coulombs: float):
        """
        Create a new instance of ElectricCharge from a value in coulombs.

        

        :param meters: The ElectricCharge value in coulombs.
        :type coulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(coulombs, ElectricChargeUnits.Coulomb)

    
    @staticmethod
    def from_ampere_hours(ampere_hours: float):
        """
        Create a new instance of ElectricCharge from a value in ampere_hours.

        

        :param meters: The ElectricCharge value in ampere_hours.
        :type ampere_hours: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(ampere_hours, ElectricChargeUnits.AmpereHour)

    
    @staticmethod
    def from_picocoulombs(picocoulombs: float):
        """
        Create a new instance of ElectricCharge from a value in picocoulombs.

        

        :param meters: The ElectricCharge value in picocoulombs.
        :type picocoulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(picocoulombs, ElectricChargeUnits.Picocoulomb)

    
    @staticmethod
    def from_nanocoulombs(nanocoulombs: float):
        """
        Create a new instance of ElectricCharge from a value in nanocoulombs.

        

        :param meters: The ElectricCharge value in nanocoulombs.
        :type nanocoulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(nanocoulombs, ElectricChargeUnits.Nanocoulomb)

    
    @staticmethod
    def from_microcoulombs(microcoulombs: float):
        """
        Create a new instance of ElectricCharge from a value in microcoulombs.

        

        :param meters: The ElectricCharge value in microcoulombs.
        :type microcoulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(microcoulombs, ElectricChargeUnits.Microcoulomb)

    
    @staticmethod
    def from_millicoulombs(millicoulombs: float):
        """
        Create a new instance of ElectricCharge from a value in millicoulombs.

        

        :param meters: The ElectricCharge value in millicoulombs.
        :type millicoulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(millicoulombs, ElectricChargeUnits.Millicoulomb)

    
    @staticmethod
    def from_kilocoulombs(kilocoulombs: float):
        """
        Create a new instance of ElectricCharge from a value in kilocoulombs.

        

        :param meters: The ElectricCharge value in kilocoulombs.
        :type kilocoulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(kilocoulombs, ElectricChargeUnits.Kilocoulomb)

    
    @staticmethod
    def from_megacoulombs(megacoulombs: float):
        """
        Create a new instance of ElectricCharge from a value in megacoulombs.

        

        :param meters: The ElectricCharge value in megacoulombs.
        :type megacoulombs: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(megacoulombs, ElectricChargeUnits.Megacoulomb)

    
    @staticmethod
    def from_milliampere_hours(milliampere_hours: float):
        """
        Create a new instance of ElectricCharge from a value in milliampere_hours.

        

        :param meters: The ElectricCharge value in milliampere_hours.
        :type milliampere_hours: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(milliampere_hours, ElectricChargeUnits.MilliampereHour)

    
    @staticmethod
    def from_kiloampere_hours(kiloampere_hours: float):
        """
        Create a new instance of ElectricCharge from a value in kiloampere_hours.

        

        :param meters: The ElectricCharge value in kiloampere_hours.
        :type kiloampere_hours: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(kiloampere_hours, ElectricChargeUnits.KiloampereHour)

    
    @staticmethod
    def from_megaampere_hours(megaampere_hours: float):
        """
        Create a new instance of ElectricCharge from a value in megaampere_hours.

        

        :param meters: The ElectricCharge value in megaampere_hours.
        :type megaampere_hours: float
        :return: A new instance of ElectricCharge.
        :rtype: ElectricCharge
        """
        return ElectricCharge(megaampere_hours, ElectricChargeUnits.MegaampereHour)

    
    @property
    def coulombs(self) -> float:
        """
        
        """
        if self.__coulombs != None:
            return self.__coulombs
        self.__coulombs = self.__convert_from_base(ElectricChargeUnits.Coulomb)
        return self.__coulombs

    
    @property
    def ampere_hours(self) -> float:
        """
        
        """
        if self.__ampere_hours != None:
            return self.__ampere_hours
        self.__ampere_hours = self.__convert_from_base(ElectricChargeUnits.AmpereHour)
        return self.__ampere_hours

    
    @property
    def picocoulombs(self) -> float:
        """
        
        """
        if self.__picocoulombs != None:
            return self.__picocoulombs
        self.__picocoulombs = self.__convert_from_base(ElectricChargeUnits.Picocoulomb)
        return self.__picocoulombs

    
    @property
    def nanocoulombs(self) -> float:
        """
        
        """
        if self.__nanocoulombs != None:
            return self.__nanocoulombs
        self.__nanocoulombs = self.__convert_from_base(ElectricChargeUnits.Nanocoulomb)
        return self.__nanocoulombs

    
    @property
    def microcoulombs(self) -> float:
        """
        
        """
        if self.__microcoulombs != None:
            return self.__microcoulombs
        self.__microcoulombs = self.__convert_from_base(ElectricChargeUnits.Microcoulomb)
        return self.__microcoulombs

    
    @property
    def millicoulombs(self) -> float:
        """
        
        """
        if self.__millicoulombs != None:
            return self.__millicoulombs
        self.__millicoulombs = self.__convert_from_base(ElectricChargeUnits.Millicoulomb)
        return self.__millicoulombs

    
    @property
    def kilocoulombs(self) -> float:
        """
        
        """
        if self.__kilocoulombs != None:
            return self.__kilocoulombs
        self.__kilocoulombs = self.__convert_from_base(ElectricChargeUnits.Kilocoulomb)
        return self.__kilocoulombs

    
    @property
    def megacoulombs(self) -> float:
        """
        
        """
        if self.__megacoulombs != None:
            return self.__megacoulombs
        self.__megacoulombs = self.__convert_from_base(ElectricChargeUnits.Megacoulomb)
        return self.__megacoulombs

    
    @property
    def milliampere_hours(self) -> float:
        """
        
        """
        if self.__milliampere_hours != None:
            return self.__milliampere_hours
        self.__milliampere_hours = self.__convert_from_base(ElectricChargeUnits.MilliampereHour)
        return self.__milliampere_hours

    
    @property
    def kiloampere_hours(self) -> float:
        """
        
        """
        if self.__kiloampere_hours != None:
            return self.__kiloampere_hours
        self.__kiloampere_hours = self.__convert_from_base(ElectricChargeUnits.KiloampereHour)
        return self.__kiloampere_hours

    
    @property
    def megaampere_hours(self) -> float:
        """
        
        """
        if self.__megaampere_hours != None:
            return self.__megaampere_hours
        self.__megaampere_hours = self.__convert_from_base(ElectricChargeUnits.MegaampereHour)
        return self.__megaampere_hours

    
    def to_string(self, unit: ElectricChargeUnits = ElectricChargeUnits.Coulomb, fractional_digits: int = None) -> str:
        """
        Format the ElectricCharge to a string.
        
        Note: the default format for ElectricCharge is Coulomb.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricCharge. Default is 'Coulomb'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricChargeUnits.Coulomb:
            return f"""{super()._truncate_fraction_digits(self.coulombs, fractional_digits)} C"""
        
        if unit == ElectricChargeUnits.AmpereHour:
            return f"""{super()._truncate_fraction_digits(self.ampere_hours, fractional_digits)} A-h"""
        
        if unit == ElectricChargeUnits.Picocoulomb:
            return f"""{super()._truncate_fraction_digits(self.picocoulombs, fractional_digits)} pC"""
        
        if unit == ElectricChargeUnits.Nanocoulomb:
            return f"""{super()._truncate_fraction_digits(self.nanocoulombs, fractional_digits)} nC"""
        
        if unit == ElectricChargeUnits.Microcoulomb:
            return f"""{super()._truncate_fraction_digits(self.microcoulombs, fractional_digits)} μC"""
        
        if unit == ElectricChargeUnits.Millicoulomb:
            return f"""{super()._truncate_fraction_digits(self.millicoulombs, fractional_digits)} mC"""
        
        if unit == ElectricChargeUnits.Kilocoulomb:
            return f"""{super()._truncate_fraction_digits(self.kilocoulombs, fractional_digits)} kC"""
        
        if unit == ElectricChargeUnits.Megacoulomb:
            return f"""{super()._truncate_fraction_digits(self.megacoulombs, fractional_digits)} MC"""
        
        if unit == ElectricChargeUnits.MilliampereHour:
            return f"""{super()._truncate_fraction_digits(self.milliampere_hours, fractional_digits)} mA-h"""
        
        if unit == ElectricChargeUnits.KiloampereHour:
            return f"""{super()._truncate_fraction_digits(self.kiloampere_hours, fractional_digits)} kA-h"""
        
        if unit == ElectricChargeUnits.MegaampereHour:
            return f"""{super()._truncate_fraction_digits(self.megaampere_hours, fractional_digits)} MA-h"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricChargeUnits = ElectricChargeUnits.Coulomb) -> str:
        """
        Get ElectricCharge unit abbreviation.
        Note! the default abbreviation for ElectricCharge is Coulomb.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricChargeUnits.Coulomb:
            return """C"""
        
        if unit_abbreviation == ElectricChargeUnits.AmpereHour:
            return """A-h"""
        
        if unit_abbreviation == ElectricChargeUnits.Picocoulomb:
            return """pC"""
        
        if unit_abbreviation == ElectricChargeUnits.Nanocoulomb:
            return """nC"""
        
        if unit_abbreviation == ElectricChargeUnits.Microcoulomb:
            return """μC"""
        
        if unit_abbreviation == ElectricChargeUnits.Millicoulomb:
            return """mC"""
        
        if unit_abbreviation == ElectricChargeUnits.Kilocoulomb:
            return """kC"""
        
        if unit_abbreviation == ElectricChargeUnits.Megacoulomb:
            return """MC"""
        
        if unit_abbreviation == ElectricChargeUnits.MilliampereHour:
            return """mA-h"""
        
        if unit_abbreviation == ElectricChargeUnits.KiloampereHour:
            return """kA-h"""
        
        if unit_abbreviation == ElectricChargeUnits.MegaampereHour:
            return """MA-h"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ReciprocalAreaUnits(Enum):
        """
            ReciprocalAreaUnits enumeration
        """
        
        InverseSquareMeter = 'InverseSquareMeter'
        """
            
        """
        
        InverseSquareKilometer = 'InverseSquareKilometer'
        """
            
        """
        
        InverseSquareDecimeter = 'InverseSquareDecimeter'
        """
            
        """
        
        InverseSquareCentimeter = 'InverseSquareCentimeter'
        """
            
        """
        
        InverseSquareMillimeter = 'InverseSquareMillimeter'
        """
            
        """
        
        InverseSquareMicrometer = 'InverseSquareMicrometer'
        """
            
        """
        
        InverseSquareMile = 'InverseSquareMile'
        """
            
        """
        
        InverseSquareYard = 'InverseSquareYard'
        """
            
        """
        
        InverseSquareFoot = 'InverseSquareFoot'
        """
            
        """
        
        InverseUsSurveySquareFoot = 'InverseUsSurveySquareFoot'
        """
            
        """
        
        InverseSquareInch = 'InverseSquareInch'
        """
            
        """
        

class ReciprocalAreaDto:
    """
    A DTO representation of a ReciprocalArea

    Attributes:
        value (float): The value of the ReciprocalArea.
        unit (ReciprocalAreaUnits): The specific unit that the ReciprocalArea value is representing.
    """

    def __init__(self, value: float, unit: ReciprocalAreaUnits):
        """
        Create a new DTO representation of a ReciprocalArea

        Parameters:
            value (float): The value of the ReciprocalArea.
            unit (ReciprocalAreaUnits): The specific unit that the ReciprocalArea value is representing.
        """
        self.value: float = value
        """
        The value of the ReciprocalArea
        """
        self.unit: ReciprocalAreaUnits = unit
        """
        The specific unit that the ReciprocalArea value is representing
        """

    def to_json(self):
        """
        Get a ReciprocalArea DTO JSON object representing the current unit.

        :return: JSON object represents ReciprocalArea DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InverseSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ReciprocalArea DTO from a json representation.

        :param data: The ReciprocalArea DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InverseSquareMeter"}
        :return: A new instance of ReciprocalAreaDto.
        :rtype: ReciprocalAreaDto
        """
        return ReciprocalAreaDto(value=data["value"], unit=ReciprocalAreaUnits(data["unit"]))


class ReciprocalArea(AbstractMeasure):
    """
    Reciprocal area (Inverse-square) quantity is used to specify a physical quantity inversely proportional to the square of the distance.

    Args:
        value (float): The value.
        from_unit (ReciprocalAreaUnits): The ReciprocalArea unit to create from, The default unit is InverseSquareMeter
    """
    def __init__(self, value: float, from_unit: ReciprocalAreaUnits = ReciprocalAreaUnits.InverseSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__inverse_square_meters = None
        
        self.__inverse_square_kilometers = None
        
        self.__inverse_square_decimeters = None
        
        self.__inverse_square_centimeters = None
        
        self.__inverse_square_millimeters = None
        
        self.__inverse_square_micrometers = None
        
        self.__inverse_square_miles = None
        
        self.__inverse_square_yards = None
        
        self.__inverse_square_feet = None
        
        self.__inverse_us_survey_square_feet = None
        
        self.__inverse_square_inches = None
        

    def convert(self, unit: ReciprocalAreaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ReciprocalAreaUnits = ReciprocalAreaUnits.InverseSquareMeter) -> ReciprocalAreaDto:
        """
        Get a new instance of ReciprocalArea DTO representing the current unit.

        :param hold_in_unit: The specific ReciprocalArea unit to store the ReciprocalArea value in the DTO representation.
        :type hold_in_unit: ReciprocalAreaUnits
        :return: A new instance of ReciprocalAreaDto.
        :rtype: ReciprocalAreaDto
        """
        return ReciprocalAreaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ReciprocalAreaUnits = ReciprocalAreaUnits.InverseSquareMeter):
        """
        Get a ReciprocalArea DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ReciprocalArea unit to store the ReciprocalArea value in the DTO representation.
        :type hold_in_unit: ReciprocalAreaUnits
        :return: JSON object represents ReciprocalArea DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InverseSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(reciprocal_area_dto: ReciprocalAreaDto):
        """
        Obtain a new instance of ReciprocalArea from a DTO unit object.

        :param reciprocal_area_dto: The ReciprocalArea DTO representation.
        :type reciprocal_area_dto: ReciprocalAreaDto
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(reciprocal_area_dto.value, reciprocal_area_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ReciprocalArea from a DTO unit json representation.

        :param data: The ReciprocalArea DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InverseSquareMeter"}
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea.from_dto(ReciprocalAreaDto.from_json(data))

    def __convert_from_base(self, from_unit: ReciprocalAreaUnits) -> float:
        value = self._value
        
        if from_unit == ReciprocalAreaUnits.InverseSquareMeter:
            return (value)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareKilometer:
            return (value * 1e6)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareDecimeter:
            return (value * 1e-2)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareCentimeter:
            return (value * 1e-4)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareMillimeter:
            return (value * 1e-6)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareMicrometer:
            return (value * 1e-12)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareMile:
            return (value * 2.59e6)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareYard:
            return (value * 0.836127)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareFoot:
            return (value * 0.092903)
        
        if from_unit == ReciprocalAreaUnits.InverseUsSurveySquareFoot:
            return (value * 0.09290341161)
        
        if from_unit == ReciprocalAreaUnits.InverseSquareInch:
            return (value * 0.00064516)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ReciprocalAreaUnits) -> float:
        
        if to_unit == ReciprocalAreaUnits.InverseSquareMeter:
            return (value)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareKilometer:
            return (value / 1e6)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareDecimeter:
            return (value / 1e-2)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareCentimeter:
            return (value / 1e-4)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareMillimeter:
            return (value / 1e-6)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareMicrometer:
            return (value / 1e-12)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareMile:
            return (value / 2.59e6)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareYard:
            return (value / 0.836127)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareFoot:
            return (value / 0.092903)
        
        if to_unit == ReciprocalAreaUnits.InverseUsSurveySquareFoot:
            return (value / 0.09290341161)
        
        if to_unit == ReciprocalAreaUnits.InverseSquareInch:
            return (value / 0.00064516)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_inverse_square_meters(inverse_square_meters: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_meters.

        

        :param meters: The ReciprocalArea value in inverse_square_meters.
        :type inverse_square_meters: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_meters, ReciprocalAreaUnits.InverseSquareMeter)

    
    @staticmethod
    def from_inverse_square_kilometers(inverse_square_kilometers: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_kilometers.

        

        :param meters: The ReciprocalArea value in inverse_square_kilometers.
        :type inverse_square_kilometers: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_kilometers, ReciprocalAreaUnits.InverseSquareKilometer)

    
    @staticmethod
    def from_inverse_square_decimeters(inverse_square_decimeters: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_decimeters.

        

        :param meters: The ReciprocalArea value in inverse_square_decimeters.
        :type inverse_square_decimeters: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_decimeters, ReciprocalAreaUnits.InverseSquareDecimeter)

    
    @staticmethod
    def from_inverse_square_centimeters(inverse_square_centimeters: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_centimeters.

        

        :param meters: The ReciprocalArea value in inverse_square_centimeters.
        :type inverse_square_centimeters: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_centimeters, ReciprocalAreaUnits.InverseSquareCentimeter)

    
    @staticmethod
    def from_inverse_square_millimeters(inverse_square_millimeters: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_millimeters.

        

        :param meters: The ReciprocalArea value in inverse_square_millimeters.
        :type inverse_square_millimeters: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_millimeters, ReciprocalAreaUnits.InverseSquareMillimeter)

    
    @staticmethod
    def from_inverse_square_micrometers(inverse_square_micrometers: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_micrometers.

        

        :param meters: The ReciprocalArea value in inverse_square_micrometers.
        :type inverse_square_micrometers: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_micrometers, ReciprocalAreaUnits.InverseSquareMicrometer)

    
    @staticmethod
    def from_inverse_square_miles(inverse_square_miles: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_miles.

        

        :param meters: The ReciprocalArea value in inverse_square_miles.
        :type inverse_square_miles: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_miles, ReciprocalAreaUnits.InverseSquareMile)

    
    @staticmethod
    def from_inverse_square_yards(inverse_square_yards: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_yards.

        

        :param meters: The ReciprocalArea value in inverse_square_yards.
        :type inverse_square_yards: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_yards, ReciprocalAreaUnits.InverseSquareYard)

    
    @staticmethod
    def from_inverse_square_feet(inverse_square_feet: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_feet.

        

        :param meters: The ReciprocalArea value in inverse_square_feet.
        :type inverse_square_feet: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_feet, ReciprocalAreaUnits.InverseSquareFoot)

    
    @staticmethod
    def from_inverse_us_survey_square_feet(inverse_us_survey_square_feet: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_us_survey_square_feet.

        

        :param meters: The ReciprocalArea value in inverse_us_survey_square_feet.
        :type inverse_us_survey_square_feet: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_us_survey_square_feet, ReciprocalAreaUnits.InverseUsSurveySquareFoot)

    
    @staticmethod
    def from_inverse_square_inches(inverse_square_inches: float):
        """
        Create a new instance of ReciprocalArea from a value in inverse_square_inches.

        

        :param meters: The ReciprocalArea value in inverse_square_inches.
        :type inverse_square_inches: float
        :return: A new instance of ReciprocalArea.
        :rtype: ReciprocalArea
        """
        return ReciprocalArea(inverse_square_inches, ReciprocalAreaUnits.InverseSquareInch)

    
    @property
    def inverse_square_meters(self) -> float:
        """
        
        """
        if self.__inverse_square_meters != None:
            return self.__inverse_square_meters
        self.__inverse_square_meters = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareMeter)
        return self.__inverse_square_meters

    
    @property
    def inverse_square_kilometers(self) -> float:
        """
        
        """
        if self.__inverse_square_kilometers != None:
            return self.__inverse_square_kilometers
        self.__inverse_square_kilometers = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareKilometer)
        return self.__inverse_square_kilometers

    
    @property
    def inverse_square_decimeters(self) -> float:
        """
        
        """
        if self.__inverse_square_decimeters != None:
            return self.__inverse_square_decimeters
        self.__inverse_square_decimeters = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareDecimeter)
        return self.__inverse_square_decimeters

    
    @property
    def inverse_square_centimeters(self) -> float:
        """
        
        """
        if self.__inverse_square_centimeters != None:
            return self.__inverse_square_centimeters
        self.__inverse_square_centimeters = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareCentimeter)
        return self.__inverse_square_centimeters

    
    @property
    def inverse_square_millimeters(self) -> float:
        """
        
        """
        if self.__inverse_square_millimeters != None:
            return self.__inverse_square_millimeters
        self.__inverse_square_millimeters = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareMillimeter)
        return self.__inverse_square_millimeters

    
    @property
    def inverse_square_micrometers(self) -> float:
        """
        
        """
        if self.__inverse_square_micrometers != None:
            return self.__inverse_square_micrometers
        self.__inverse_square_micrometers = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareMicrometer)
        return self.__inverse_square_micrometers

    
    @property
    def inverse_square_miles(self) -> float:
        """
        
        """
        if self.__inverse_square_miles != None:
            return self.__inverse_square_miles
        self.__inverse_square_miles = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareMile)
        return self.__inverse_square_miles

    
    @property
    def inverse_square_yards(self) -> float:
        """
        
        """
        if self.__inverse_square_yards != None:
            return self.__inverse_square_yards
        self.__inverse_square_yards = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareYard)
        return self.__inverse_square_yards

    
    @property
    def inverse_square_feet(self) -> float:
        """
        
        """
        if self.__inverse_square_feet != None:
            return self.__inverse_square_feet
        self.__inverse_square_feet = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareFoot)
        return self.__inverse_square_feet

    
    @property
    def inverse_us_survey_square_feet(self) -> float:
        """
        
        """
        if self.__inverse_us_survey_square_feet != None:
            return self.__inverse_us_survey_square_feet
        self.__inverse_us_survey_square_feet = self.__convert_from_base(ReciprocalAreaUnits.InverseUsSurveySquareFoot)
        return self.__inverse_us_survey_square_feet

    
    @property
    def inverse_square_inches(self) -> float:
        """
        
        """
        if self.__inverse_square_inches != None:
            return self.__inverse_square_inches
        self.__inverse_square_inches = self.__convert_from_base(ReciprocalAreaUnits.InverseSquareInch)
        return self.__inverse_square_inches

    
    def to_string(self, unit: ReciprocalAreaUnits = ReciprocalAreaUnits.InverseSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the ReciprocalArea to a string.
        
        Note: the default format for ReciprocalArea is InverseSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ReciprocalArea. Default is 'InverseSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ReciprocalAreaUnits.InverseSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_meters, fractional_digits)} m⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareKilometer:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_kilometers, fractional_digits)} km⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_decimeters, fractional_digits)} dm⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_centimeters, fractional_digits)} cm⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_millimeters, fractional_digits)} mm⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareMicrometer:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_micrometers, fractional_digits)} µm⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareMile:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_miles, fractional_digits)} mi⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareYard:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_yards, fractional_digits)} yd⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_feet, fractional_digits)} ft⁻²"""
        
        if unit == ReciprocalAreaUnits.InverseUsSurveySquareFoot:
            return f"""{super()._truncate_fraction_digits(self.inverse_us_survey_square_feet, fractional_digits)} ft⁻² (US)"""
        
        if unit == ReciprocalAreaUnits.InverseSquareInch:
            return f"""{super()._truncate_fraction_digits(self.inverse_square_inches, fractional_digits)} in⁻²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ReciprocalAreaUnits = ReciprocalAreaUnits.InverseSquareMeter) -> str:
        """
        Get ReciprocalArea unit abbreviation.
        Note! the default abbreviation for ReciprocalArea is InverseSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareMeter:
            return """m⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareKilometer:
            return """km⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareDecimeter:
            return """dm⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareCentimeter:
            return """cm⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareMillimeter:
            return """mm⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareMicrometer:
            return """µm⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareMile:
            return """mi⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareYard:
            return """yd⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareFoot:
            return """ft⁻²"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseUsSurveySquareFoot:
            return """ft⁻² (US)"""
        
        if unit_abbreviation == ReciprocalAreaUnits.InverseSquareInch:
            return """in⁻²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class JerkUnits(Enum):
        """
            JerkUnits enumeration
        """
        
        MeterPerSecondCubed = 'MeterPerSecondCubed'
        """
            
        """
        
        InchPerSecondCubed = 'InchPerSecondCubed'
        """
            
        """
        
        FootPerSecondCubed = 'FootPerSecondCubed'
        """
            
        """
        
        StandardGravitiesPerSecond = 'StandardGravitiesPerSecond'
        """
            
        """
        
        NanometerPerSecondCubed = 'NanometerPerSecondCubed'
        """
            
        """
        
        MicrometerPerSecondCubed = 'MicrometerPerSecondCubed'
        """
            
        """
        
        MillimeterPerSecondCubed = 'MillimeterPerSecondCubed'
        """
            
        """
        
        CentimeterPerSecondCubed = 'CentimeterPerSecondCubed'
        """
            
        """
        
        DecimeterPerSecondCubed = 'DecimeterPerSecondCubed'
        """
            
        """
        
        KilometerPerSecondCubed = 'KilometerPerSecondCubed'
        """
            
        """
        
        MillistandardGravitiesPerSecond = 'MillistandardGravitiesPerSecond'
        """
            
        """
        

class JerkDto:
    """
    A DTO representation of a Jerk

    Attributes:
        value (float): The value of the Jerk.
        unit (JerkUnits): The specific unit that the Jerk value is representing.
    """

    def __init__(self, value: float, unit: JerkUnits):
        """
        Create a new DTO representation of a Jerk

        Parameters:
            value (float): The value of the Jerk.
            unit (JerkUnits): The specific unit that the Jerk value is representing.
        """
        self.value: float = value
        """
        The value of the Jerk
        """
        self.unit: JerkUnits = unit
        """
        The specific unit that the Jerk value is representing
        """

    def to_json(self):
        """
        Get a Jerk DTO JSON object representing the current unit.

        :return: JSON object represents Jerk DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MeterPerSecondCubed"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Jerk DTO from a json representation.

        :param data: The Jerk DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MeterPerSecondCubed"}
        :return: A new instance of JerkDto.
        :rtype: JerkDto
        """
        return JerkDto(value=data["value"], unit=JerkUnits(data["unit"]))


class Jerk(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (JerkUnits): The Jerk unit to create from, The default unit is MeterPerSecondCubed
    """
    def __init__(self, value: float, from_unit: JerkUnits = JerkUnits.MeterPerSecondCubed):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__meters_per_second_cubed = None
        
        self.__inches_per_second_cubed = None
        
        self.__feet_per_second_cubed = None
        
        self.__standard_gravities_per_second = None
        
        self.__nanometers_per_second_cubed = None
        
        self.__micrometers_per_second_cubed = None
        
        self.__millimeters_per_second_cubed = None
        
        self.__centimeters_per_second_cubed = None
        
        self.__decimeters_per_second_cubed = None
        
        self.__kilometers_per_second_cubed = None
        
        self.__millistandard_gravities_per_second = None
        

    def convert(self, unit: JerkUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: JerkUnits = JerkUnits.MeterPerSecondCubed) -> JerkDto:
        """
        Get a new instance of Jerk DTO representing the current unit.

        :param hold_in_unit: The specific Jerk unit to store the Jerk value in the DTO representation.
        :type hold_in_unit: JerkUnits
        :return: A new instance of JerkDto.
        :rtype: JerkDto
        """
        return JerkDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: JerkUnits = JerkUnits.MeterPerSecondCubed):
        """
        Get a Jerk DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Jerk unit to store the Jerk value in the DTO representation.
        :type hold_in_unit: JerkUnits
        :return: JSON object represents Jerk DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MeterPerSecondCubed"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(jerk_dto: JerkDto):
        """
        Obtain a new instance of Jerk from a DTO unit object.

        :param jerk_dto: The Jerk DTO representation.
        :type jerk_dto: JerkDto
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(jerk_dto.value, jerk_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Jerk from a DTO unit json representation.

        :param data: The Jerk DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MeterPerSecondCubed"}
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk.from_dto(JerkDto.from_json(data))

    def __convert_from_base(self, from_unit: JerkUnits) -> float:
        value = self._value
        
        if from_unit == JerkUnits.MeterPerSecondCubed:
            return (value)
        
        if from_unit == JerkUnits.InchPerSecondCubed:
            return (value / 0.0254)
        
        if from_unit == JerkUnits.FootPerSecondCubed:
            return (value / 0.304800)
        
        if from_unit == JerkUnits.StandardGravitiesPerSecond:
            return (value / 9.80665)
        
        if from_unit == JerkUnits.NanometerPerSecondCubed:
            return ((value) / 1e-09)
        
        if from_unit == JerkUnits.MicrometerPerSecondCubed:
            return ((value) / 1e-06)
        
        if from_unit == JerkUnits.MillimeterPerSecondCubed:
            return ((value) / 0.001)
        
        if from_unit == JerkUnits.CentimeterPerSecondCubed:
            return ((value) / 0.01)
        
        if from_unit == JerkUnits.DecimeterPerSecondCubed:
            return ((value) / 0.1)
        
        if from_unit == JerkUnits.KilometerPerSecondCubed:
            return ((value) / 1000.0)
        
        if from_unit == JerkUnits.MillistandardGravitiesPerSecond:
            return ((value / 9.80665) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: JerkUnits) -> float:
        
        if to_unit == JerkUnits.MeterPerSecondCubed:
            return (value)
        
        if to_unit == JerkUnits.InchPerSecondCubed:
            return (value * 0.0254)
        
        if to_unit == JerkUnits.FootPerSecondCubed:
            return (value * 0.304800)
        
        if to_unit == JerkUnits.StandardGravitiesPerSecond:
            return (value * 9.80665)
        
        if to_unit == JerkUnits.NanometerPerSecondCubed:
            return ((value) * 1e-09)
        
        if to_unit == JerkUnits.MicrometerPerSecondCubed:
            return ((value) * 1e-06)
        
        if to_unit == JerkUnits.MillimeterPerSecondCubed:
            return ((value) * 0.001)
        
        if to_unit == JerkUnits.CentimeterPerSecondCubed:
            return ((value) * 0.01)
        
        if to_unit == JerkUnits.DecimeterPerSecondCubed:
            return ((value) * 0.1)
        
        if to_unit == JerkUnits.KilometerPerSecondCubed:
            return ((value) * 1000.0)
        
        if to_unit == JerkUnits.MillistandardGravitiesPerSecond:
            return ((value * 9.80665) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_meters_per_second_cubed(meters_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in meters_per_second_cubed.

        

        :param meters: The Jerk value in meters_per_second_cubed.
        :type meters_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(meters_per_second_cubed, JerkUnits.MeterPerSecondCubed)

    
    @staticmethod
    def from_inches_per_second_cubed(inches_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in inches_per_second_cubed.

        

        :param meters: The Jerk value in inches_per_second_cubed.
        :type inches_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(inches_per_second_cubed, JerkUnits.InchPerSecondCubed)

    
    @staticmethod
    def from_feet_per_second_cubed(feet_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in feet_per_second_cubed.

        

        :param meters: The Jerk value in feet_per_second_cubed.
        :type feet_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(feet_per_second_cubed, JerkUnits.FootPerSecondCubed)

    
    @staticmethod
    def from_standard_gravities_per_second(standard_gravities_per_second: float):
        """
        Create a new instance of Jerk from a value in standard_gravities_per_second.

        

        :param meters: The Jerk value in standard_gravities_per_second.
        :type standard_gravities_per_second: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(standard_gravities_per_second, JerkUnits.StandardGravitiesPerSecond)

    
    @staticmethod
    def from_nanometers_per_second_cubed(nanometers_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in nanometers_per_second_cubed.

        

        :param meters: The Jerk value in nanometers_per_second_cubed.
        :type nanometers_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(nanometers_per_second_cubed, JerkUnits.NanometerPerSecondCubed)

    
    @staticmethod
    def from_micrometers_per_second_cubed(micrometers_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in micrometers_per_second_cubed.

        

        :param meters: The Jerk value in micrometers_per_second_cubed.
        :type micrometers_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(micrometers_per_second_cubed, JerkUnits.MicrometerPerSecondCubed)

    
    @staticmethod
    def from_millimeters_per_second_cubed(millimeters_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in millimeters_per_second_cubed.

        

        :param meters: The Jerk value in millimeters_per_second_cubed.
        :type millimeters_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(millimeters_per_second_cubed, JerkUnits.MillimeterPerSecondCubed)

    
    @staticmethod
    def from_centimeters_per_second_cubed(centimeters_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in centimeters_per_second_cubed.

        

        :param meters: The Jerk value in centimeters_per_second_cubed.
        :type centimeters_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(centimeters_per_second_cubed, JerkUnits.CentimeterPerSecondCubed)

    
    @staticmethod
    def from_decimeters_per_second_cubed(decimeters_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in decimeters_per_second_cubed.

        

        :param meters: The Jerk value in decimeters_per_second_cubed.
        :type decimeters_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(decimeters_per_second_cubed, JerkUnits.DecimeterPerSecondCubed)

    
    @staticmethod
    def from_kilometers_per_second_cubed(kilometers_per_second_cubed: float):
        """
        Create a new instance of Jerk from a value in kilometers_per_second_cubed.

        

        :param meters: The Jerk value in kilometers_per_second_cubed.
        :type kilometers_per_second_cubed: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(kilometers_per_second_cubed, JerkUnits.KilometerPerSecondCubed)

    
    @staticmethod
    def from_millistandard_gravities_per_second(millistandard_gravities_per_second: float):
        """
        Create a new instance of Jerk from a value in millistandard_gravities_per_second.

        

        :param meters: The Jerk value in millistandard_gravities_per_second.
        :type millistandard_gravities_per_second: float
        :return: A new instance of Jerk.
        :rtype: Jerk
        """
        return Jerk(millistandard_gravities_per_second, JerkUnits.MillistandardGravitiesPerSecond)

    
    @property
    def meters_per_second_cubed(self) -> float:
        """
        
        """
        if self.__meters_per_second_cubed != None:
            return self.__meters_per_second_cubed
        self.__meters_per_second_cubed = self.__convert_from_base(JerkUnits.MeterPerSecondCubed)
        return self.__meters_per_second_cubed

    
    @property
    def inches_per_second_cubed(self) -> float:
        """
        
        """
        if self.__inches_per_second_cubed != None:
            return self.__inches_per_second_cubed
        self.__inches_per_second_cubed = self.__convert_from_base(JerkUnits.InchPerSecondCubed)
        return self.__inches_per_second_cubed

    
    @property
    def feet_per_second_cubed(self) -> float:
        """
        
        """
        if self.__feet_per_second_cubed != None:
            return self.__feet_per_second_cubed
        self.__feet_per_second_cubed = self.__convert_from_base(JerkUnits.FootPerSecondCubed)
        return self.__feet_per_second_cubed

    
    @property
    def standard_gravities_per_second(self) -> float:
        """
        
        """
        if self.__standard_gravities_per_second != None:
            return self.__standard_gravities_per_second
        self.__standard_gravities_per_second = self.__convert_from_base(JerkUnits.StandardGravitiesPerSecond)
        return self.__standard_gravities_per_second

    
    @property
    def nanometers_per_second_cubed(self) -> float:
        """
        
        """
        if self.__nanometers_per_second_cubed != None:
            return self.__nanometers_per_second_cubed
        self.__nanometers_per_second_cubed = self.__convert_from_base(JerkUnits.NanometerPerSecondCubed)
        return self.__nanometers_per_second_cubed

    
    @property
    def micrometers_per_second_cubed(self) -> float:
        """
        
        """
        if self.__micrometers_per_second_cubed != None:
            return self.__micrometers_per_second_cubed
        self.__micrometers_per_second_cubed = self.__convert_from_base(JerkUnits.MicrometerPerSecondCubed)
        return self.__micrometers_per_second_cubed

    
    @property
    def millimeters_per_second_cubed(self) -> float:
        """
        
        """
        if self.__millimeters_per_second_cubed != None:
            return self.__millimeters_per_second_cubed
        self.__millimeters_per_second_cubed = self.__convert_from_base(JerkUnits.MillimeterPerSecondCubed)
        return self.__millimeters_per_second_cubed

    
    @property
    def centimeters_per_second_cubed(self) -> float:
        """
        
        """
        if self.__centimeters_per_second_cubed != None:
            return self.__centimeters_per_second_cubed
        self.__centimeters_per_second_cubed = self.__convert_from_base(JerkUnits.CentimeterPerSecondCubed)
        return self.__centimeters_per_second_cubed

    
    @property
    def decimeters_per_second_cubed(self) -> float:
        """
        
        """
        if self.__decimeters_per_second_cubed != None:
            return self.__decimeters_per_second_cubed
        self.__decimeters_per_second_cubed = self.__convert_from_base(JerkUnits.DecimeterPerSecondCubed)
        return self.__decimeters_per_second_cubed

    
    @property
    def kilometers_per_second_cubed(self) -> float:
        """
        
        """
        if self.__kilometers_per_second_cubed != None:
            return self.__kilometers_per_second_cubed
        self.__kilometers_per_second_cubed = self.__convert_from_base(JerkUnits.KilometerPerSecondCubed)
        return self.__kilometers_per_second_cubed

    
    @property
    def millistandard_gravities_per_second(self) -> float:
        """
        
        """
        if self.__millistandard_gravities_per_second != None:
            return self.__millistandard_gravities_per_second
        self.__millistandard_gravities_per_second = self.__convert_from_base(JerkUnits.MillistandardGravitiesPerSecond)
        return self.__millistandard_gravities_per_second

    
    def to_string(self, unit: JerkUnits = JerkUnits.MeterPerSecondCubed, fractional_digits: int = None) -> str:
        """
        Format the Jerk to a string.
        
        Note: the default format for Jerk is MeterPerSecondCubed.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Jerk. Default is 'MeterPerSecondCubed'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == JerkUnits.MeterPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.meters_per_second_cubed, fractional_digits)} m/s³"""
        
        if unit == JerkUnits.InchPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.inches_per_second_cubed, fractional_digits)} in/s³"""
        
        if unit == JerkUnits.FootPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.feet_per_second_cubed, fractional_digits)} ft/s³"""
        
        if unit == JerkUnits.StandardGravitiesPerSecond:
            return f"""{super()._truncate_fraction_digits(self.standard_gravities_per_second, fractional_digits)} g/s"""
        
        if unit == JerkUnits.NanometerPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.nanometers_per_second_cubed, fractional_digits)} nm/s³"""
        
        if unit == JerkUnits.MicrometerPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.micrometers_per_second_cubed, fractional_digits)} μm/s³"""
        
        if unit == JerkUnits.MillimeterPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.millimeters_per_second_cubed, fractional_digits)} mm/s³"""
        
        if unit == JerkUnits.CentimeterPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.centimeters_per_second_cubed, fractional_digits)} cm/s³"""
        
        if unit == JerkUnits.DecimeterPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.decimeters_per_second_cubed, fractional_digits)} dm/s³"""
        
        if unit == JerkUnits.KilometerPerSecondCubed:
            return f"""{super()._truncate_fraction_digits(self.kilometers_per_second_cubed, fractional_digits)} km/s³"""
        
        if unit == JerkUnits.MillistandardGravitiesPerSecond:
            return f"""{super()._truncate_fraction_digits(self.millistandard_gravities_per_second, fractional_digits)} mg/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: JerkUnits = JerkUnits.MeterPerSecondCubed) -> str:
        """
        Get Jerk unit abbreviation.
        Note! the default abbreviation for Jerk is MeterPerSecondCubed.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == JerkUnits.MeterPerSecondCubed:
            return """m/s³"""
        
        if unit_abbreviation == JerkUnits.InchPerSecondCubed:
            return """in/s³"""
        
        if unit_abbreviation == JerkUnits.FootPerSecondCubed:
            return """ft/s³"""
        
        if unit_abbreviation == JerkUnits.StandardGravitiesPerSecond:
            return """g/s"""
        
        if unit_abbreviation == JerkUnits.NanometerPerSecondCubed:
            return """nm/s³"""
        
        if unit_abbreviation == JerkUnits.MicrometerPerSecondCubed:
            return """μm/s³"""
        
        if unit_abbreviation == JerkUnits.MillimeterPerSecondCubed:
            return """mm/s³"""
        
        if unit_abbreviation == JerkUnits.CentimeterPerSecondCubed:
            return """cm/s³"""
        
        if unit_abbreviation == JerkUnits.DecimeterPerSecondCubed:
            return """dm/s³"""
        
        if unit_abbreviation == JerkUnits.KilometerPerSecondCubed:
            return """km/s³"""
        
        if unit_abbreviation == JerkUnits.MillistandardGravitiesPerSecond:
            return """mg/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class EntropyUnits(Enum):
        """
            EntropyUnits enumeration
        """
        
        JoulePerKelvin = 'JoulePerKelvin'
        """
            
        """
        
        CaloriePerKelvin = 'CaloriePerKelvin'
        """
            
        """
        
        JoulePerDegreeCelsius = 'JoulePerDegreeCelsius'
        """
            
        """
        
        KilojoulePerKelvin = 'KilojoulePerKelvin'
        """
            
        """
        
        MegajoulePerKelvin = 'MegajoulePerKelvin'
        """
            
        """
        
        KilocaloriePerKelvin = 'KilocaloriePerKelvin'
        """
            
        """
        
        KilojoulePerDegreeCelsius = 'KilojoulePerDegreeCelsius'
        """
            
        """
        

class EntropyDto:
    """
    A DTO representation of a Entropy

    Attributes:
        value (float): The value of the Entropy.
        unit (EntropyUnits): The specific unit that the Entropy value is representing.
    """

    def __init__(self, value: float, unit: EntropyUnits):
        """
        Create a new DTO representation of a Entropy

        Parameters:
            value (float): The value of the Entropy.
            unit (EntropyUnits): The specific unit that the Entropy value is representing.
        """
        self.value: float = value
        """
        The value of the Entropy
        """
        self.unit: EntropyUnits = unit
        """
        The specific unit that the Entropy value is representing
        """

    def to_json(self):
        """
        Get a Entropy DTO JSON object representing the current unit.

        :return: JSON object represents Entropy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerKelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Entropy DTO from a json representation.

        :param data: The Entropy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerKelvin"}
        :return: A new instance of EntropyDto.
        :rtype: EntropyDto
        """
        return EntropyDto(value=data["value"], unit=EntropyUnits(data["unit"]))


class Entropy(AbstractMeasure):
    """
    Entropy is an important concept in the branch of science known as thermodynamics. The idea of "irreversibility" is central to the understanding of entropy.  It is often said that entropy is an expression of the disorder, or randomness of a system, or of our lack of information about it. Entropy is an extensive property. It has the dimension of energy divided by temperature, which has a unit of joules per kelvin (J/K) in the International System of Units

    Args:
        value (float): The value.
        from_unit (EntropyUnits): The Entropy unit to create from, The default unit is JoulePerKelvin
    """
    def __init__(self, value: float, from_unit: EntropyUnits = EntropyUnits.JoulePerKelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_kelvin = None
        
        self.__calories_per_kelvin = None
        
        self.__joules_per_degree_celsius = None
        
        self.__kilojoules_per_kelvin = None
        
        self.__megajoules_per_kelvin = None
        
        self.__kilocalories_per_kelvin = None
        
        self.__kilojoules_per_degree_celsius = None
        

    def convert(self, unit: EntropyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: EntropyUnits = EntropyUnits.JoulePerKelvin) -> EntropyDto:
        """
        Get a new instance of Entropy DTO representing the current unit.

        :param hold_in_unit: The specific Entropy unit to store the Entropy value in the DTO representation.
        :type hold_in_unit: EntropyUnits
        :return: A new instance of EntropyDto.
        :rtype: EntropyDto
        """
        return EntropyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: EntropyUnits = EntropyUnits.JoulePerKelvin):
        """
        Get a Entropy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Entropy unit to store the Entropy value in the DTO representation.
        :type hold_in_unit: EntropyUnits
        :return: JSON object represents Entropy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerKelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(entropy_dto: EntropyDto):
        """
        Obtain a new instance of Entropy from a DTO unit object.

        :param entropy_dto: The Entropy DTO representation.
        :type entropy_dto: EntropyDto
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(entropy_dto.value, entropy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Entropy from a DTO unit json representation.

        :param data: The Entropy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerKelvin"}
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy.from_dto(EntropyDto.from_json(data))

    def __convert_from_base(self, from_unit: EntropyUnits) -> float:
        value = self._value
        
        if from_unit == EntropyUnits.JoulePerKelvin:
            return (value)
        
        if from_unit == EntropyUnits.CaloriePerKelvin:
            return (value / 4.184)
        
        if from_unit == EntropyUnits.JoulePerDegreeCelsius:
            return (value)
        
        if from_unit == EntropyUnits.KilojoulePerKelvin:
            return ((value) / 1000.0)
        
        if from_unit == EntropyUnits.MegajoulePerKelvin:
            return ((value) / 1000000.0)
        
        if from_unit == EntropyUnits.KilocaloriePerKelvin:
            return ((value / 4.184) / 1000.0)
        
        if from_unit == EntropyUnits.KilojoulePerDegreeCelsius:
            return ((value) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: EntropyUnits) -> float:
        
        if to_unit == EntropyUnits.JoulePerKelvin:
            return (value)
        
        if to_unit == EntropyUnits.CaloriePerKelvin:
            return (value * 4.184)
        
        if to_unit == EntropyUnits.JoulePerDegreeCelsius:
            return (value)
        
        if to_unit == EntropyUnits.KilojoulePerKelvin:
            return ((value) * 1000.0)
        
        if to_unit == EntropyUnits.MegajoulePerKelvin:
            return ((value) * 1000000.0)
        
        if to_unit == EntropyUnits.KilocaloriePerKelvin:
            return ((value * 4.184) * 1000.0)
        
        if to_unit == EntropyUnits.KilojoulePerDegreeCelsius:
            return ((value) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_kelvin(joules_per_kelvin: float):
        """
        Create a new instance of Entropy from a value in joules_per_kelvin.

        

        :param meters: The Entropy value in joules_per_kelvin.
        :type joules_per_kelvin: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(joules_per_kelvin, EntropyUnits.JoulePerKelvin)

    
    @staticmethod
    def from_calories_per_kelvin(calories_per_kelvin: float):
        """
        Create a new instance of Entropy from a value in calories_per_kelvin.

        

        :param meters: The Entropy value in calories_per_kelvin.
        :type calories_per_kelvin: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(calories_per_kelvin, EntropyUnits.CaloriePerKelvin)

    
    @staticmethod
    def from_joules_per_degree_celsius(joules_per_degree_celsius: float):
        """
        Create a new instance of Entropy from a value in joules_per_degree_celsius.

        

        :param meters: The Entropy value in joules_per_degree_celsius.
        :type joules_per_degree_celsius: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(joules_per_degree_celsius, EntropyUnits.JoulePerDegreeCelsius)

    
    @staticmethod
    def from_kilojoules_per_kelvin(kilojoules_per_kelvin: float):
        """
        Create a new instance of Entropy from a value in kilojoules_per_kelvin.

        

        :param meters: The Entropy value in kilojoules_per_kelvin.
        :type kilojoules_per_kelvin: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(kilojoules_per_kelvin, EntropyUnits.KilojoulePerKelvin)

    
    @staticmethod
    def from_megajoules_per_kelvin(megajoules_per_kelvin: float):
        """
        Create a new instance of Entropy from a value in megajoules_per_kelvin.

        

        :param meters: The Entropy value in megajoules_per_kelvin.
        :type megajoules_per_kelvin: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(megajoules_per_kelvin, EntropyUnits.MegajoulePerKelvin)

    
    @staticmethod
    def from_kilocalories_per_kelvin(kilocalories_per_kelvin: float):
        """
        Create a new instance of Entropy from a value in kilocalories_per_kelvin.

        

        :param meters: The Entropy value in kilocalories_per_kelvin.
        :type kilocalories_per_kelvin: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(kilocalories_per_kelvin, EntropyUnits.KilocaloriePerKelvin)

    
    @staticmethod
    def from_kilojoules_per_degree_celsius(kilojoules_per_degree_celsius: float):
        """
        Create a new instance of Entropy from a value in kilojoules_per_degree_celsius.

        

        :param meters: The Entropy value in kilojoules_per_degree_celsius.
        :type kilojoules_per_degree_celsius: float
        :return: A new instance of Entropy.
        :rtype: Entropy
        """
        return Entropy(kilojoules_per_degree_celsius, EntropyUnits.KilojoulePerDegreeCelsius)

    
    @property
    def joules_per_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_kelvin != None:
            return self.__joules_per_kelvin
        self.__joules_per_kelvin = self.__convert_from_base(EntropyUnits.JoulePerKelvin)
        return self.__joules_per_kelvin

    
    @property
    def calories_per_kelvin(self) -> float:
        """
        
        """
        if self.__calories_per_kelvin != None:
            return self.__calories_per_kelvin
        self.__calories_per_kelvin = self.__convert_from_base(EntropyUnits.CaloriePerKelvin)
        return self.__calories_per_kelvin

    
    @property
    def joules_per_degree_celsius(self) -> float:
        """
        
        """
        if self.__joules_per_degree_celsius != None:
            return self.__joules_per_degree_celsius
        self.__joules_per_degree_celsius = self.__convert_from_base(EntropyUnits.JoulePerDegreeCelsius)
        return self.__joules_per_degree_celsius

    
    @property
    def kilojoules_per_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_kelvin != None:
            return self.__kilojoules_per_kelvin
        self.__kilojoules_per_kelvin = self.__convert_from_base(EntropyUnits.KilojoulePerKelvin)
        return self.__kilojoules_per_kelvin

    
    @property
    def megajoules_per_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_kelvin != None:
            return self.__megajoules_per_kelvin
        self.__megajoules_per_kelvin = self.__convert_from_base(EntropyUnits.MegajoulePerKelvin)
        return self.__megajoules_per_kelvin

    
    @property
    def kilocalories_per_kelvin(self) -> float:
        """
        
        """
        if self.__kilocalories_per_kelvin != None:
            return self.__kilocalories_per_kelvin
        self.__kilocalories_per_kelvin = self.__convert_from_base(EntropyUnits.KilocaloriePerKelvin)
        return self.__kilocalories_per_kelvin

    
    @property
    def kilojoules_per_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilojoules_per_degree_celsius != None:
            return self.__kilojoules_per_degree_celsius
        self.__kilojoules_per_degree_celsius = self.__convert_from_base(EntropyUnits.KilojoulePerDegreeCelsius)
        return self.__kilojoules_per_degree_celsius

    
    def to_string(self, unit: EntropyUnits = EntropyUnits.JoulePerKelvin, fractional_digits: int = None) -> str:
        """
        Format the Entropy to a string.
        
        Note: the default format for Entropy is JoulePerKelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Entropy. Default is 'JoulePerKelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == EntropyUnits.JoulePerKelvin:
            return f"""{super()._truncate_fraction_digits(self.joules_per_kelvin, fractional_digits)} J/K"""
        
        if unit == EntropyUnits.CaloriePerKelvin:
            return f"""{super()._truncate_fraction_digits(self.calories_per_kelvin, fractional_digits)} cal/K"""
        
        if unit == EntropyUnits.JoulePerDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.joules_per_degree_celsius, fractional_digits)} J/C"""
        
        if unit == EntropyUnits.KilojoulePerKelvin:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_kelvin, fractional_digits)} kJ/K"""
        
        if unit == EntropyUnits.MegajoulePerKelvin:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_kelvin, fractional_digits)} MJ/K"""
        
        if unit == EntropyUnits.KilocaloriePerKelvin:
            return f"""{super()._truncate_fraction_digits(self.kilocalories_per_kelvin, fractional_digits)} kcal/K"""
        
        if unit == EntropyUnits.KilojoulePerDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_degree_celsius, fractional_digits)} kJ/C"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: EntropyUnits = EntropyUnits.JoulePerKelvin) -> str:
        """
        Get Entropy unit abbreviation.
        Note! the default abbreviation for Entropy is JoulePerKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == EntropyUnits.JoulePerKelvin:
            return """J/K"""
        
        if unit_abbreviation == EntropyUnits.CaloriePerKelvin:
            return """cal/K"""
        
        if unit_abbreviation == EntropyUnits.JoulePerDegreeCelsius:
            return """J/C"""
        
        if unit_abbreviation == EntropyUnits.KilojoulePerKelvin:
            return """kJ/K"""
        
        if unit_abbreviation == EntropyUnits.MegajoulePerKelvin:
            return """MJ/K"""
        
        if unit_abbreviation == EntropyUnits.KilocaloriePerKelvin:
            return """kcal/K"""
        
        if unit_abbreviation == EntropyUnits.KilojoulePerDegreeCelsius:
            return """kJ/C"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class PowerRatioUnits(Enum):
        """
            PowerRatioUnits enumeration
        """
        
        DecibelWatt = 'DecibelWatt'
        """
            
        """
        
        DecibelMilliwatt = 'DecibelMilliwatt'
        """
            
        """
        

class PowerRatioDto:
    """
    A DTO representation of a PowerRatio

    Attributes:
        value (float): The value of the PowerRatio.
        unit (PowerRatioUnits): The specific unit that the PowerRatio value is representing.
    """

    def __init__(self, value: float, unit: PowerRatioUnits):
        """
        Create a new DTO representation of a PowerRatio

        Parameters:
            value (float): The value of the PowerRatio.
            unit (PowerRatioUnits): The specific unit that the PowerRatio value is representing.
        """
        self.value: float = value
        """
        The value of the PowerRatio
        """
        self.unit: PowerRatioUnits = unit
        """
        The specific unit that the PowerRatio value is representing
        """

    def to_json(self):
        """
        Get a PowerRatio DTO JSON object representing the current unit.

        :return: JSON object represents PowerRatio DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecibelWatt"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of PowerRatio DTO from a json representation.

        :param data: The PowerRatio DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecibelWatt"}
        :return: A new instance of PowerRatioDto.
        :rtype: PowerRatioDto
        """
        return PowerRatioDto(value=data["value"], unit=PowerRatioUnits(data["unit"]))


class PowerRatio(AbstractMeasure):
    """
    The strength of a signal expressed in decibels (dB) relative to one watt.

    Args:
        value (float): The value.
        from_unit (PowerRatioUnits): The PowerRatio unit to create from, The default unit is DecibelWatt
    """
    def __init__(self, value: float, from_unit: PowerRatioUnits = PowerRatioUnits.DecibelWatt):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__decibel_watts = None
        
        self.__decibel_milliwatts = None
        

    def convert(self, unit: PowerRatioUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: PowerRatioUnits = PowerRatioUnits.DecibelWatt) -> PowerRatioDto:
        """
        Get a new instance of PowerRatio DTO representing the current unit.

        :param hold_in_unit: The specific PowerRatio unit to store the PowerRatio value in the DTO representation.
        :type hold_in_unit: PowerRatioUnits
        :return: A new instance of PowerRatioDto.
        :rtype: PowerRatioDto
        """
        return PowerRatioDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: PowerRatioUnits = PowerRatioUnits.DecibelWatt):
        """
        Get a PowerRatio DTO JSON object representing the current unit.

        :param hold_in_unit: The specific PowerRatio unit to store the PowerRatio value in the DTO representation.
        :type hold_in_unit: PowerRatioUnits
        :return: JSON object represents PowerRatio DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecibelWatt"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(power_ratio_dto: PowerRatioDto):
        """
        Obtain a new instance of PowerRatio from a DTO unit object.

        :param power_ratio_dto: The PowerRatio DTO representation.
        :type power_ratio_dto: PowerRatioDto
        :return: A new instance of PowerRatio.
        :rtype: PowerRatio
        """
        return PowerRatio(power_ratio_dto.value, power_ratio_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of PowerRatio from a DTO unit json representation.

        :param data: The PowerRatio DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecibelWatt"}
        :return: A new instance of PowerRatio.
        :rtype: PowerRatio
        """
        return PowerRatio.from_dto(PowerRatioDto.from_json(data))

    def __convert_from_base(self, from_unit: PowerRatioUnits) -> float:
        value = self._value
        
        if from_unit == PowerRatioUnits.DecibelWatt:
            return (value)
        
        if from_unit == PowerRatioUnits.DecibelMilliwatt:
            return (value + 30)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PowerRatioUnits) -> float:
        
        if to_unit == PowerRatioUnits.DecibelWatt:
            return (value)
        
        if to_unit == PowerRatioUnits.DecibelMilliwatt:
            return (value - 30)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_decibel_watts(decibel_watts: float):
        """
        Create a new instance of PowerRatio from a value in decibel_watts.

        

        :param meters: The PowerRatio value in decibel_watts.
        :type decibel_watts: float
        :return: A new instance of PowerRatio.
        :rtype: PowerRatio
        """
        return PowerRatio(decibel_watts, PowerRatioUnits.DecibelWatt)

    
    @staticmethod
    def from_decibel_milliwatts(decibel_milliwatts: float):
        """
        Create a new instance of PowerRatio from a value in decibel_milliwatts.

        

        :param meters: The PowerRatio value in decibel_milliwatts.
        :type decibel_milliwatts: float
        :return: A new instance of PowerRatio.
        :rtype: PowerRatio
        """
        return PowerRatio(decibel_milliwatts, PowerRatioUnits.DecibelMilliwatt)

    
    @property
    def decibel_watts(self) -> float:
        """
        
        """
        if self.__decibel_watts != None:
            return self.__decibel_watts
        self.__decibel_watts = self.__convert_from_base(PowerRatioUnits.DecibelWatt)
        return self.__decibel_watts

    
    @property
    def decibel_milliwatts(self) -> float:
        """
        
        """
        if self.__decibel_milliwatts != None:
            return self.__decibel_milliwatts
        self.__decibel_milliwatts = self.__convert_from_base(PowerRatioUnits.DecibelMilliwatt)
        return self.__decibel_milliwatts

    
    def to_string(self, unit: PowerRatioUnits = PowerRatioUnits.DecibelWatt, fractional_digits: int = None) -> str:
        """
        Format the PowerRatio to a string.
        
        Note: the default format for PowerRatio is DecibelWatt.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the PowerRatio. Default is 'DecibelWatt'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == PowerRatioUnits.DecibelWatt:
            return f"""{super()._truncate_fraction_digits(self.decibel_watts, fractional_digits)} dBW"""
        
        if unit == PowerRatioUnits.DecibelMilliwatt:
            return f"""{super()._truncate_fraction_digits(self.decibel_milliwatts, fractional_digits)} dBmW"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: PowerRatioUnits = PowerRatioUnits.DecibelWatt) -> str:
        """
        Get PowerRatio unit abbreviation.
        Note! the default abbreviation for PowerRatio is DecibelWatt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PowerRatioUnits.DecibelWatt:
            return """dBW"""
        
        if unit_abbreviation == PowerRatioUnits.DecibelMilliwatt:
            return """dBmW"""
        